@@ -9,7 +9,7 @@
 
 #include "adm_locl.h"
 
-RCSID("$Id: ext_srvtab.c,v 1.16 1997/12/11 18:03:08 bg Exp $");
+RCSID("$Id: ext_srvtab.c,v 1.18 1999/09/16 20:37:20 assar Exp $");
 
 static des_cblock master_key;
 static des_cblock session_key;
@@ -73,7 +73,7 @@ main(int argc, char **argv)
 		if (++i >= argc)
 		    usage();
 		else {
-		    strcpy(realm, argv[i]);
+		    strlcpy(realm, argv[i], REALM_SZ);
 		    /* 
 		     * This is to humor the broken way commandline
 		     * argument parsing is done.  Later, this
