@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -43,6 +38,7 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
+#include <ctype.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -55,7 +51,7 @@
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
-#if defined(HAVE_SYS_IOCTL_H) && SunOS != 4
+#if defined(HAVE_SYS_IOCTL_H) && SunOS != 40
 #include <sys/ioctl.h>
 #endif
 #ifdef HAVE_SYS_IOCCOM_H
@@ -70,7 +66,7 @@
 
 #include <roken.h>
 
-RCSID("$Id: aklog.c,v 1.18 1997/10/24 10:15:41 assar Exp $");
+RCSID("$Id: aklog.c,v 1.24 1999/12/02 16:58:28 joda Exp $");
 
 static int debug = 0;
 
@@ -104,7 +100,8 @@ expand_cell_name(char *cell)
     return cell;
   while(fgets(buf, sizeof(buf), f) != NULL) {
     if(buf[0] == '>') {
-      for(p=buf; *p && *p != ' ' && *p != '\t' && *p != '\n'; p++);
+      for(p=buf; *p && !isspace(*p) && *p != '#'; p++)
+	  ;
       *p = '\0';
       if(strstr(buf, cell)){
 	fclose(f);
@@ -135,6 +132,7 @@ createuser (char *cell)
       err (1, "open(%s)", _PATH_THISCELL);
     if (fgets (cellbuf, sizeof(cellbuf), f) == NULL)
       err (1, "read cellname from %s", _PATH_THISCELL);
+    fclose (f);
     len = strlen(cellbuf);
     if (cellbuf[len-1] == '\n')
       cellbuf[len-1] = '\0';
@@ -171,14 +169,14 @@ main(int argc, char **argv)
     if(!strncmp(argv[i], "-createuser", 11)){
       do_createuser = do_aklog = 1;
 
-    }else if(!strncmp(argv[i], "-c", 2)){
+    }else if(!strncmp(argv[i], "-c", 2) && i + 1 < argc){
       cell = expand_cell_name(argv[++i]);
       do_aklog = 1;
 
-    }else if(!strncmp(argv[i], "-k", 2)){
+    }else if(!strncmp(argv[i], "-k", 2) && i + 1 < argc){
       realm = argv[++i];
 
-    }else if(!strncmp(argv[i], "-p", 2)){
+    }else if(!strncmp(argv[i], "-p", 2) && i + 1 < argc){
       if(k_afs_cell_of_file(argv[++i], cellbuf, sizeof(cellbuf)))
 	errx (1, "No cell found for file \"%s\".", argv[i]);
       else
@@ -219,7 +217,7 @@ main(int argc, char **argv)
     }
     if(do_aklog == 1){
       do_aklog = 0;
-      if(k_afsklog(cell, realm))
+      if(krb_afslog(cell, realm))
 	errx (1, "Failed getting tokens for cell %s in realm %s.", 
 	      cell?cell:"(local cell)", realm?realm:"(local realm)");
     }
@@ -229,7 +227,7 @@ main(int argc, char **argv)
 	errx (1, "Failed creating user in cell %s", cell?cell:"(local cell)");
     }
   }
-  if(do_aklog == -1 && do_createuser == -1 && k_afsklog(0, realm))
+  if(do_aklog == -1 && do_createuser == -1 && krb_afslog(0, realm))
     errx (1, "Failed getting tokens for cell %s in realm %s.", 
 	  cell?cell:"(local cell)", realm?realm:"(local realm)");
   return 0;
