@@ -33,7 +33,7 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Id: rsh.c,v 1.37 1997/11/03 11:18:14 assar Exp $");
+RCSID("$Id: rsh.c,v 1.43 1999/11/13 06:13:34 assar Exp $");
 
 CREDENTIALS cred;
 Key_schedule schedule;
@@ -49,7 +49,7 @@ static void
 usage(void)
 {
     fprintf(stderr,
-	    "usage: rsh [-ndKx] [-k realm] [-l login] host [command]\n");
+	    "usage: rsh [-ndKx] [-k realm] [-p port] [-l login] host [command]\n");
     exit(1);
 }
 
@@ -63,11 +63,13 @@ copyargs(char **argv)
     cc = 0;
     for (ap = argv; *ap; ++ap)
 	cc += strlen(*ap) + 1;
-    if (!(args = malloc(cc))) 
+    args = malloc(cc);
+    if (args == NULL)
 	errx(1, "Out of memory.");
     for (p = args, ap = argv; *ap; ++ap) {
 	strcpy(p, *ap);
-	for (p = strcpy(p, *ap); *p; ++p);
+	while(*p)
+	    ++p;
 	if (ap[1])
 	    *p++ = ' ';
     }
@@ -92,7 +94,7 @@ talk(int nflag, sigset_t omask, int pid, int rem)
     int cc, wc;
     char *bp;
     fd_set readfrom, ready, rembits;
-    char buf[BUFSIZ];
+    char buf[DES_RW_MAXWRITE];
 
     if (pid == 0) {
 	if (nflag)
@@ -184,7 +186,7 @@ int
 main(int argc, char **argv)
 {
     struct passwd *pw;
-    int sv_port;
+    int sv_port, user_port = 0;
     sigset_t omask;
     int argoff, ch, dflag, nflag, nfork, one, pid, rem, uid;
     char *args, *host, *user, *local_user;
@@ -197,13 +199,13 @@ main(int argc, char **argv)
     set_progname(argv[0]);
 
     /* handle "rsh host flags" */
-    if (!host && argc > 2 && argv[1][0] != '-') {
+    if (argc > 2 && argv[1][0] != '-') {
 	host = argv[1];
 	argoff = 1;
     }
 
-#define	OPTIONS	"+8KLde:k:l:nwx"
-    while ((ch = getopt(argc - argoff, argv + argoff, OPTIONS)) != EOF)
+#define	OPTIONS	"+8KLde:k:l:np:wx"
+    while ((ch = getopt(argc - argoff, argv + argoff, OPTIONS)) != -1)
 	switch(ch) {
 	case 'K':
 	    use_kerberos = 0;
@@ -221,7 +223,7 @@ main(int argc, char **argv)
 	    break;
 	case 'k':
 	    dest_realm = dst_realm_buf;
-	    strncpy(dest_realm, optarg, REALM_SZ);
+	    strlcpy(dest_realm, optarg, REALM_SZ);
 	    break;
 	case 'n':
 	    nflag = nfork = 1;
@@ -229,6 +231,15 @@ main(int argc, char **argv)
 	case 'x':
 	    doencrypt = 1;
 	    break;
+	case 'p': {
+	    char *endptr;
+
+	    user_port = strtol (optarg, &endptr, 0);
+	    if (user_port == 0 && optarg == endptr)
+		errx (1, "Bad port `%s'", optarg);
+	    user_port = htons(user_port);
+	    break;
+	}
 	case '?':
 	default:
 	    usage();
@@ -265,7 +276,10 @@ main(int argc, char **argv)
 
     args = copyargs(argv+optind);
 
-    sv_port=get_shell_port(use_kerberos, doencrypt);
+    if (user_port)
+	sv_port = user_port;
+    else
+	sv_port = get_shell_port(use_kerberos, doencrypt);
 
     if (use_kerberos) {
 	setuid(getuid());
@@ -281,7 +295,7 @@ main(int argc, char **argv)
 	    rem = krcmd(&host, sv_port, user, args, &rfd2,
 			dest_realm);
 	if (rem < 0) {
-	    int i;
+	    int i = 0;
 	    char **newargv;
 
 	    if (errno == ECONNREFUSED)
@@ -291,10 +305,15 @@ main(int argc, char **argv)
 	    newargv = malloc((argc + 2) * sizeof(*newargv));
 	    if (newargv == NULL)
 		err(1, "malloc");
-	    newargv[0] = argv[0];
-	    newargv[1] = "-K";
-	    for(i = 1; i < argc; ++i)
-		newargv[i + 1] = argv[i];
+	    newargv[i] = argv[i];
+	    ++i;
+	    if (argv[i][0] != '-') {
+		newargv[i] = argv[i];
+		++i;
+	    }
+	    newargv[i++] = "-K";
+	    for(; i <= argc; ++i)
+		newargv[i] = argv[i - 1];
 	    newargv[argc + 1] = NULL;
 	    execv(_PATH_RSH, newargv);
 	}
