@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995-1999 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,9 +33,51 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Id: utmp_login.c,v 1.9 1996/11/19 22:48:24 assar Exp $");
+RCSID("$Id: utmp_login.c,v 1.16 1999/12/02 16:58:29 joda Exp $");
 
-#ifdef UTMPX_DOES_UTMP_LOGGING
+#ifdef HAVE_UTMP_H
+void
+prepare_utmp (struct utmp *utmp, char *tty, char *username, char *hostname)
+{
+    char *ttyx = clean_ttyname (tty);
+
+    memset(utmp, 0, sizeof(*utmp));
+    utmp->ut_time = time(NULL);
+    strncpy(utmp->ut_line, ttyx, sizeof(utmp->ut_line));
+    strncpy(utmp->ut_name, username, sizeof(utmp->ut_name));
+
+# ifdef HAVE_STRUCT_UTMP_UT_USER
+    strncpy(utmp->ut_user, username, sizeof(utmp->ut_user));
+# endif
+
+# ifdef HAVE_STRUCT_UTMP_UT_ADDR
+    if (hostname[0]) {
+        struct hostent *he;
+	if ((he = gethostbyname(hostname)))
+	    memcpy(&utmp->ut_addr, he->h_addr_list[0],
+		   sizeof(utmp->ut_addr));
+    }
+# endif
+
+# ifdef HAVE_STRUCT_UTMP_UT_HOST
+    strncpy(utmp->ut_host, hostname, sizeof(utmp->ut_host));
+# endif
+
+# ifdef HAVE_STRUCT_UTMP_UT_TYPE
+    utmp->ut_type = USER_PROCESS;
+# endif
+
+# ifdef HAVE_STRUCT_UTMP_UT_PID
+    utmp->ut_pid = getpid();
+# endif
+
+# ifdef HAVE_STRUCT_UTMP_UT_ID
+    strncpy(utmp->ut_id, make_id(ttyx), sizeof(utmp->ut_id));
+# endif
+}
+#endif
+
+#ifdef HAVE_UTMPX_H
 void utmp_login(char *tty, char *username, char *hostname) { return; }
 #else
 
@@ -49,54 +86,9 @@ void utmp_login(char *tty, char *username, char *hostname) { return; }
 void utmp_login(char *tty, char *username, char *hostname)
 {
     struct utmp utmp;
-    struct hostent *he;
     int fd;
 
-    char *ttyx; /* tty w/o /dev/* */
-    char *id;
-
-    ttyx = tty;
-
-    if(strncmp(tty, "/dev/", 5) == 0)
-	ttyx = tty + 5;
-
-    memset(&utmp, 0, sizeof(utmp));
-    utmp.ut_time = time(NULL);
-    strncpy(utmp.ut_line, ttyx, sizeof(utmp.ut_line));
-    strncpy(utmp.ut_name, username, sizeof(utmp.ut_name));
-
-# ifdef HAVE_UT_USER
-    strncpy(utmp.ut_user, username, sizeof(utmp.ut_user));
-# endif
-
-# ifdef HAVE_UT_ADDR
-    if (hostname[0]) {
-	if ((he = gethostbyname(hostname)))
-	    memcpy(&utmp.ut_addr, he->h_addr_list[0],
-		   sizeof(utmp.ut_addr));
-    }
-# endif
-
-# ifdef HAVE_UT_HOST
-    strncpy(utmp.ut_host, hostname, sizeof(utmp.ut_host));
-# endif
-
-# ifdef HAVE_UT_TYPE
-    utmp.ut_type = USER_PROCESS;
-# endif
-
-# ifdef HAVE_UT_PID
-    utmp.ut_pid = getpid();
-# endif
-
-# ifdef HAVE_UT_ID
-    /* any particular reason to not include "tty" ? */
-    id = ttyx;
-    if(strncmp(ttyx, "tty", 3) == 0)
-	id += 3;
-    strncpy(utmp.ut_id, id, sizeof(utmp.ut_id));
-# endif
-
+    prepare_utmp (&utmp, tty, username, hostname);
 
 #ifdef HAVE_SETUTENT
     utmpname(_PATH_UTMP);
@@ -123,4 +115,4 @@ void utmp_login(char *tty, char *username, char *hostname)
 	close(fd);
     }
 }
-#endif /* !UTMPX_DOES_UTMP_LOGGING */
+#endif /* !HAVE_UTMPX_H */
