@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,14 +33,12 @@
 
 #include "kauth.h"
 
-RCSID("$Id: kauthd.c,v 1.15 1996/11/17 21:08:27 assar Exp $");
+RCSID("$Id: kauthd.c,v 1.25 1999/12/02 16:58:31 joda Exp $");
 
-static char name[SNAME_SZ + 1];
-static char inst[INST_SZ + 1];
-static char realm[REALM_SZ + 1];
-static char locuser[SNAME_SZ + 1];
+krb_principal princ;
+static char locuser[SNAME_SZ];
 static int  lifetime;
-static char tktfile[MaxPathLen + 1];
+static char tktfile[MaxPathLen];
 
 struct remote_args {
      int sock;
@@ -55,8 +48,12 @@ struct remote_args {
 };
 
 static int
-decrypt_remote_tkt (char *user, char *inst, char *realm, void *varg,
-		    key_proc_t key_proc, KTEXT *cipp)
+decrypt_remote_tkt (const char *user,
+		    const char *inst,
+		    const char *realm,
+		    const void *varg,
+		    key_proc_t key_proc,
+		    KTEXT *cipp)
 {
      char buf[BUFSIZ];
      void *ptr;
@@ -80,7 +77,7 @@ doit(int sock)
      int status;
      KTEXT_ST ticket;
      AUTH_DAT auth;
-     char instance[INST_SZ + 1];
+     char instance[INST_SZ];
      des_key_schedule schedule;
      struct sockaddr_in thisaddr, thataddr;
      int addrlen;
@@ -88,7 +85,7 @@ doit(int sock)
      char buf[BUFSIZ];
      void *data;
      struct passwd *passwd;
-     char version[KRB_SENDAUTH_VLEN];
+     char version[KRB_SENDAUTH_VLEN + 1];
      char remotehost[MaxHostNameLen];
 
      addrlen = sizeof(thisaddr);
@@ -104,12 +101,12 @@ doit(int sock)
 
      inaddr2str (thataddr.sin_addr, remotehost, sizeof(remotehost));
 
-     k_getsockinst (sock, instance);
+     k_getsockinst (sock, instance, sizeof(instance));
      status = krb_recvauth (KOPT_DO_MUTUAL, sock, &ticket, "rcmd", instance,
 			    &thataddr, &thisaddr, &auth, "", schedule,
 			    version);
      if (status != KSUCCESS ||
-	 strncmp(version, "RKINIT.0", KRB_SENDAUTH_VLEN) != 0) {
+	 strncmp(version, KAUTH_VERSION, KRB_SENDAUTH_VLEN) != 0) {
 	  return 1;
      }
      len = read_encrypted (sock, buf, sizeof(buf), &data, schedule,
@@ -120,7 +117,7 @@ doit(int sock)
 			   &auth.session, &thisaddr, &thataddr);
 	  return 1;
      }
-     if (unpack_args(data, name, inst, realm, &lifetime, locuser,
+     if (unpack_args(data, &princ, &lifetime, locuser,
 		     tktfile)) {
 	  write_encrypted (sock, "unpack_args failed",
 			   sizeof("unpack_args failed") - 1, schedule,
@@ -129,16 +126,17 @@ doit(int sock)
      }
 
      if( kuserok(&auth, locuser) != 0) {
-	  syslog (LOG_ERR, "%s cannot get tickets for %s%s%s@%s",
-		  locuser, name, *inst == '\0' ? "" : ".",
-		  inst, realm);
-	  write_encrypted (sock, buf, strlen(buf), schedule,
-			   &auth.session, &thisaddr, &thataddr);
-	  return 1;
+	 snprintf(buf, sizeof(buf), "%s cannot get tickets for %s",
+		  locuser, krb_unparse_name(&princ));
+	 syslog (LOG_ERR, buf);
+	 write_encrypted (sock, buf, strlen(buf), schedule,
+			  &auth.session, &thisaddr, &thataddr);
+	 return 1;
      }
      passwd = k_getpwnam (locuser);
      if (passwd == NULL) {
-	  syslog (LOG_ERR, "No user '%s'", locuser);
+	  snprintf (buf, sizeof(buf), "No user '%s'", locuser);
+	  syslog (LOG_ERR, buf);
 	  write_encrypted (sock, buf, strlen(buf), schedule,
 			   &auth.session, &thisaddr, &thataddr);
 	  return 1;
@@ -146,7 +144,8 @@ doit(int sock)
      if (setgid (passwd->pw_gid) ||
 	 initgroups(passwd->pw_name, passwd->pw_gid) ||
 	 setuid(passwd->pw_uid)) {
-	  syslog (LOG_ERR, "Could not change user");
+	  snprintf (buf, sizeof(buf), "Could not change user");
+	  syslog (LOG_ERR, buf);
 	  write_encrypted (sock, buf, strlen(buf), schedule,
 			   &auth.session, &thisaddr, &thataddr);
 	  return 1;
@@ -155,7 +154,7 @@ doit(int sock)
 		      &auth.session, &thisaddr, &thataddr);
 
      if (*tktfile == 0)
-	 sprintf(tktfile, "%s%u", TKT_ROOT, (unsigned)getuid());
+	 snprintf(tktfile, sizeof(tktfile), "%s%u", TKT_ROOT, (unsigned)getuid());
      krb_set_tkt_string (tktfile);
 
      {
@@ -167,21 +166,23 @@ doit(int sock)
 	  arg.me       = &thisaddr;
 	  arg.her      = &thataddr;
 
-	  status = krb_get_in_tkt (name, inst, realm,
+	  status = krb_get_in_tkt (princ.name, princ.instance, princ.realm,
 				   KRB_TICKET_GRANTING_TICKET,
-				   realm,
+				   princ.realm,
 				   lifetime, NULL, decrypt_remote_tkt, &arg);
      }
      if (status == KSUCCESS) {
-          syslog (LOG_INFO, "from %s: %s -> %s%s%s@%s", 
-		  remotehost,
-		  locuser, name,
-		  *inst == '\0' ? "" : ".", inst, realm);
+	 syslog (LOG_INFO, "from %s(%s): %s -> %s",
+		 remotehost,
+		 inet_ntoa(thataddr.sin_addr),
+		 locuser,
+		 krb_unparse_name (&princ));
 	  write_encrypted (sock, "ok", sizeof("ok") - 1, schedule,
 			   &auth.session, &thisaddr, &thataddr);
 	  return 0;
      } else {
-	  syslog (LOG_NOTICE, "TGT failed: %s", krb_get_err_text(status));
+	  snprintf (buf, sizeof(buf), "TGT failed: %s", krb_get_err_text(status));
+	  syslog (LOG_NOTICE, buf);
 	  write_encrypted (sock, buf, strlen(buf), schedule,
 			   &auth.session, &thisaddr, &thataddr);
 	  return 1;
