@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #include "kx.h"
 
-RCSID("$Id: kx.c,v 1.46 1998/05/28 15:20:33 joda Exp $");
+RCSID("$Id: kx.c,v 1.63 1999/12/02 16:58:32 joda Exp $");
 
 static int nchild;
 static int donep;
@@ -93,43 +88,38 @@ usr2handler (int sig)
 }
 
 /*
- * Establish authenticated connection
+ * Establish authenticated connection.  Return socket or -1.
  */
 
 static int
-connect_host (char *host, char *user, des_cblock *key,
-	      des_key_schedule schedule, int port,
-	      struct sockaddr_in *thisaddr,
-	      struct sockaddr_in *thataddr)
+connect_host (kx_context *kc)
 {
-     CREDENTIALS cred;
-     KTEXT_ST text;
-     MSG_DAT msg;
-     int status;
      int addrlen;
      struct hostent *hostent;
      int s;
      char **p;
+     struct sockaddr_in thisaddr;
+     struct sockaddr_in thataddr;
 
-     hostent = gethostbyname (host);
+     hostent = gethostbyname (kc->host);
      if (hostent == NULL) {
-	 warnx ("gethostbyname '%s' failed: %s", host,
+	 warnx ("gethostbyname '%s' failed: %s", kc->host,
 		hstrerror(h_errno));
 	 return -1;
      }
 
-     memset (thataddr, 0, sizeof(*thataddr));
-     thataddr->sin_family = AF_INET;
-     thataddr->sin_port   = port;
+     memset (&thataddr, 0, sizeof(thataddr));
+     thataddr.sin_family = AF_INET;
+     thataddr.sin_port   = kc->port;
      for(p = hostent->h_addr_list; *p; ++p) {
-	 memcpy (&thataddr->sin_addr, *p, sizeof(thataddr->sin_addr));
+	 memcpy (&thataddr.sin_addr, *p, sizeof(thataddr.sin_addr));
 
 	 s = socket (AF_INET, SOCK_STREAM, 0);
 	 if (s < 0)
 	     err (1, "socket");
 
-	 if (connect (s, (struct sockaddr *)thataddr, sizeof(*thataddr)) < 0) {
-	     warn ("connect(%s)", host);
+	 if (connect (s, (struct sockaddr *)&thataddr, sizeof(thataddr)) < 0) {
+	     warn ("connect(%s)", kc->host);
 	     close (s);
 	     continue;
 	 } else {
@@ -139,47 +129,46 @@ connect_host (char *host, char *user, des_cblock *key,
      if (*p == NULL)
 	 return -1;
 
-     addrlen = sizeof(*thisaddr);
-     if (getsockname (s, (struct sockaddr *)thisaddr, &addrlen) < 0 ||
-	 addrlen != sizeof(*thisaddr))
-	 err(1, "getsockname(%s)", host);
-     status = krb_sendauth (KOPT_DO_MUTUAL, s, &text, "rcmd",
-			    host, krb_realmofhost (host),
-			    getpid(), &msg, &cred, schedule,
-			    thisaddr, thataddr, KX_VERSION);
-     if (status != KSUCCESS) {
-	 warnx ("%s: %s\n", host, krb_get_err_text(status));
+     addrlen = sizeof(thisaddr);
+     if (getsockname (s, (struct sockaddr *)&thisaddr, &addrlen) < 0 ||
+	 addrlen != sizeof(thisaddr))
+	 err(1, "getsockname(%s)", kc->host);
+     kc->thisaddr = thisaddr;
+     kc->thataddr = thataddr;
+     if ((*kc->authenticate)(kc, s))
 	 return -1;
-     }
-     memcpy(key, cred.session, sizeof(des_cblock));
      return s;
 }
 
 /*
  * Get rid of the cookie that we were sent and get the correct one
- * from our own cookie file instead.
+ * from our own cookie file instead and then just copy data in both
+ * directions.
  */
 
 static int
-passive_session (int xserver, int fd, des_cblock *iv,
-		 des_key_schedule schedule)
+passive_session (int xserver, int fd, kx_context *kc)
 {
-    if (replace_cookie (xserver, fd, XauFileName()))
+    if (replace_cookie (xserver, fd, XauFileName(), 1))
 	return 1;
     else
-	return copy_encrypted (xserver, fd, iv, schedule);
+	return copy_encrypted (kc, xserver, fd);
 }
 
 static int
-active_session (int xserver, int fd, des_cblock *iv,
-		des_key_schedule schedule)
+active_session (int xserver, int fd, kx_context *kc)
 {
-    if (verify_and_remove_cookies (xserver, fd))
+    if (verify_and_remove_cookies (xserver, fd, 1))
 	return 1;
     else
-	return copy_encrypted (xserver, fd, iv, schedule);
+	return copy_encrypted (kc, xserver, fd);
 }
 
+/*
+ * fork (unless debugp) and print the output that will be used by the
+ * script to capture the display, xauth cookie and pid.
+ */
+
 static void
 status_output (int debugp)
 {
@@ -201,31 +190,27 @@ status_output (int debugp)
 }
 
 /*
- * Obtain an authenticated connection to `host' on `port'.  Send a kx
- * message saying we are `user' and want to use passive mode.  Wait
- * for answer on that connection and fork of a child for every new
+ * Obtain an authenticated connection on `kc'.  Send a kx message
+ * saying we are `kc->user' and want to use passive mode.  Wait for
+ * answer on that connection and fork of a child for every new
  * connection we have to make.
  */
 
 static int
-doit_passive (char *host, char *user, int debugp, int keepalivep,
-	      int port)
+doit_passive (kx_context *kc)
 {
-     des_key_schedule schedule;
-     des_cblock key;
      int otherside;
-     struct sockaddr_in me, him;
      u_char msg[1024], *p;
      int len;
-     void *ret;
      u_int32_t tmp;
+     const char *host = kc->host;
+
+     otherside = connect_host (kc);
 
-     otherside = connect_host (host, user, &key, schedule, port,
-			       &me, &him);
      if (otherside < 0)
 	 return 1;
 #if defined(SO_KEEPALIVE) && defined(HAVE_SETSOCKOPT)
-     if (keepalivep) {
+     if (kc->keepalive_flag) {
 	 int one = 1;
 
 	 setsockopt (otherside, SOL_SOCKET, SO_KEEPALIVE, (void *)&one,
@@ -235,22 +220,20 @@ doit_passive (char *host, char *user, int debugp, int keepalivep,
 
      p = msg;
      *p++ = INIT;
-     len = strlen(user);
-     p += krb_put_int (len, p, 4);
-     strncpy(p, user, len);
+     len = strlen(kc->user);
+     p += KRB_PUT_INT (len, p, sizeof(msg) - 1, 4);
+     memcpy(p, kc->user, len);
      p += len;
-     *p++ = PASSIVE | (keepalivep ? KEEP_ALIVE : 0);
-     if (write_encrypted (otherside, msg, p - msg, schedule,
-			  &key, &me, &him) < 0)
+     *p++ = PASSIVE | (kc->keepalive_flag ? KEEP_ALIVE : 0);
+     if (kx_write (kc, otherside, msg, p - msg) != p - msg)
 	 err (1, "write to %s", host);
-     len = read_encrypted (otherside, msg, sizeof(msg), &ret,
-			   schedule, &key, &him, &me);
+     len = kx_read (kc, otherside, msg, sizeof(msg));
      if (len <= 0)
 	 errx (1,
 	       "error reading initial message from %s: "
 	       "this probably means it's using an old version.",
 	       host);
-     p = (u_char *)ret;
+     p = (u_char *)msg;
      if (*p == ERROR) {
 	 p++;
 	 p += krb_get_int (p, &tmp, 4, 0);
@@ -260,27 +243,26 @@ doit_passive (char *host, char *user, int debugp, int keepalivep,
      } else
 	 p++;
      p += krb_get_int (p, &tmp, 4, 0);
-     strncpy(display, p, tmp);
+     memcpy(display, p, tmp);
      display[tmp] = '\0';
      p += tmp;
 
      p += krb_get_int (p, &tmp, 4, 0);
-     strncpy(xauthfile, p, tmp);
+     memcpy(xauthfile, p, tmp);
      xauthfile[tmp] = '\0';
      p += tmp;
 
-     status_output (debugp);
+     status_output (kc->debug_flag);
      for (;;) {
 	 pid_t child;
 
-	 len = read_encrypted (otherside, msg, sizeof(msg), &ret,
-			       schedule, &key, &him, &me);
+	 len = kx_read (kc, otherside, msg, sizeof(msg));
 	 if (len < 0)
 	     err (1, "read from %s", host);
 	 else if (len == 0)
 	     return 0;
 
-	 p = (u_char *)ret;
+	 p = (u_char *)msg;
 	 if (*p == ERROR) {
 	     p++;
 	     p += krb_get_int (p, &tmp, 4, 0);
@@ -302,7 +284,7 @@ doit_passive (char *host, char *user, int debugp, int keepalivep,
 	     int fd;
 	     int xserver;
 
-	     addr = him;
+	     addr = kc->thataddr;
 	     close (otherside);
 
 	     addr.sin_port = htons(tmp);
@@ -318,7 +300,7 @@ doit_passive (char *host, char *user, int debugp, int keepalivep,
 	     }
 #endif
 #if defined(SO_KEEPALIVE) && defined(HAVE_SETSOCKOPT)
-	     if (keepalivep) {
+	     if (kc->keepalive_flag) {
 		 int one = 1;
 
 		 setsockopt (fd, SOL_SOCKET, SO_KEEPALIVE, (void *)&one,
@@ -328,10 +310,22 @@ doit_passive (char *host, char *user, int debugp, int keepalivep,
 
 	     if (connect (fd, (struct sockaddr *)&addr, sizeof(addr)) < 0)
 		 err(1, "connect(%s)", host);
-	     xserver = connect_local_xsocket (0);
-	     if (xserver < 0)
-		 return 1;
-	     return passive_session (xserver, fd, &key, schedule);
+	     {
+		 int d = 0;
+		 char *s;
+
+		 s = getenv ("DISPLAY");
+		 if (s != NULL) {
+		     s = strchr (s, ':');
+		     if (s != NULL)
+			 d = atoi (s + 1);
+		 }
+
+		 xserver = connect_local_xsocket (d);
+		 if (xserver < 0)
+		     return 1;
+	     }
+	     return passive_session (xserver, fd, kc);
 	 } else {
 	 }
      }
@@ -342,29 +336,26 @@ doit_passive (char *host, char *user, int debugp, int keepalivep,
  */
 
 static int
-doit_active (char *host, char *user,
-	     int debugpp, int keepalivep, int tcpp, int port)
+doit_active (kx_context *kc)
 {
-    des_key_schedule schedule;
-    des_cblock key;
     int otherside;
     int nsockets;
     struct x_socket *sockets;
-    struct sockaddr_in me, him;
     u_char msg[1024], *p;
-    int len = strlen(user);
-    void *ret;
-    u_int32_t tmp;
-    int tmp2;
+    int len = strlen(kc->user);
+    int tmp, tmp2;
     char *s;
     int i;
+    size_t rem;
+    u_int32_t other_port;
+    int error;
+    const char *host = kc->host;
 
-    otherside = connect_host (host, user, &key, schedule, port,
-			      &me, &him);
+    otherside = connect_host (kc);
     if (otherside < 0)
 	return 1;
 #if defined(SO_KEEPALIVE) && defined(HAVE_SETSOCKOPT)
-    if (keepalivep) {
+    if (kc->keepalive_flag) {
 	int one = 1;
 
 	setsockopt (otherside, SOL_SOCKET, SO_KEEPALIVE, (void *)&one,
@@ -372,58 +363,80 @@ doit_active (char *host, char *user,
     }
 #endif
     p = msg;
+    rem = sizeof(msg);
     *p++ = INIT;
-    len = strlen(user);
-    p += krb_put_int (len, p, 4);
-    strncpy(p, user, len);
+    --rem;
+    len = strlen(kc->user);
+    tmp = KRB_PUT_INT (len, p, rem, 4);
+    if (tmp < 0)
+	return 1;
+    p += tmp;
+    rem -= tmp;
+    memcpy(p, kc->user, len);
     p += len;
-    *p++ = (keepalivep ? KEEP_ALIVE : 0);
+    rem -= len;
+    *p++ = (kc->keepalive_flag ? KEEP_ALIVE : 0);
+    --rem;
 
     s = getenv("DISPLAY");
     if (s == NULL || (s = strchr(s, ':')) == NULL) 
 	s = ":0";
     len = strlen (s);
-    p += krb_put_int (len, p, 4);
-    strncpy (p, s, len);
+    tmp = KRB_PUT_INT (len, p, rem, 4);
+    if (tmp < 0)
+	return 1;
+    rem -= tmp;
+    p += tmp;
+    memcpy (p, s, len);
     p += len;
+    rem -= len;
 
     s = getenv("XAUTHORITY");
     if (s == NULL)
 	s = "";
     len = strlen (s);
-    p += krb_put_int (len, p, 4);
-    strncpy (p, s, len);
+    tmp = KRB_PUT_INT (len, p, rem, 4);
+    if (tmp < 0)
+	return 1;
     p += len;
+    rem -= len;
+    memcpy (p, s, len);
+    p += len;
+    rem -= len;
 
-    if (write_encrypted (otherside, msg, p - msg, schedule,
-			 &key, &me, &him) < 0)
+    if (kx_write (kc, otherside, msg, p - msg) != p - msg)
 	err (1, "write to %s", host);
 
-    len = read_encrypted (otherside, msg, sizeof(msg), &ret,
-			  schedule, &key, &him, &me);
+    len = kx_read (kc, otherside, msg, sizeof(msg));
     if (len < 0)
 	err (1, "read from %s", host);
-    p = (u_char *)ret;
+    p = (u_char *)msg;
     if (*p == ERROR) {
+	u_int32_t u32;
+
 	p++;
-	p += krb_get_int (p, &tmp, 4, 0);
-	errx (1, "%s: %.*s", host, (int)tmp, p);
+	p += krb_get_int (p, &u32, 4, 0);
+	errx (1, "%s: %.*s", host, (int)u32, p);
     } else if (*p != ACK) {
 	errx (1, "%s: strange msg %d", host, *p);
     } else
 	p++;
 
-    tmp2 = get_xsockets (&nsockets, &sockets, tcpp);
+    tmp2 = get_xsockets (&nsockets, &sockets, kc->tcp_flag);
     if (tmp2 < 0)
 	return 1;
     display_num = tmp2;
-    if (tcpp)
+    if (kc->tcp_flag)
 	snprintf (display, display_size, "localhost:%u", display_num);
     else
 	snprintf (display, display_size, ":%u", display_num);
-    if (create_and_write_cookie (xauthfile, xauthfile_size, cookie, cookie_len))
+    error = create_and_write_cookie (xauthfile, xauthfile_size,
+				     cookie, cookie_len);
+    if (error) {
+	warnx ("failed creating cookie file: %s", strerror(error));
 	return 1;
-    status_output (debugpp);
+    }
+    status_output (kc->debug_flag);
     for (;;) {
 	fd_set fdset;
 	pid_t child;
@@ -441,31 +454,32 @@ doit_active (char *host, char *user,
 		break;
 	    }
 	fd = accept (thisfd, NULL, &zero);
-	if (fd < 0)
+	if (fd < 0) {
 	    if (errno == EINTR)
 		continue;
 	    else
 		err(1, "accept");
+	}
 
 	p = msg;
 	*p++ = NEW_CONN;
-	if (write_encrypted (otherside, msg, p - msg, schedule,
-			     &key, &me, &him) < 0)
+	if (kx_write (kc, otherside, msg, p - msg) != p - msg)
 	    err (1, "write to %s", host);
-	len = read_encrypted (otherside, msg, sizeof(msg), &ret,
-			      schedule, &key, &him, &me);
+	len = kx_read (kc, otherside, msg, sizeof(msg));
 	if (len < 0)
 	    err (1, "read from %s", host);
-	p = (u_char *)ret;
+	p = (u_char *)msg;
 	if (*p == ERROR) {
+	    u_int32_t val;
+
 	    p++;
-	    p += krb_get_int (p, &tmp, 4, 0);
-	    errx (1, "%s: %.*s", host, (int)tmp, p);
+	    p += krb_get_int (p, &val, 4, 0);
+	    errx (1, "%s: %.*s", host, (int)val, p);
 	} else if (*p != NEW_CONN) {
 	    errx (1, "%s: strange msg %d", host, *p);
 	} else {
 	    p++;
-	    p += krb_get_int (p, &tmp, 4, 0);
+	    p += krb_get_int (p, &other_port, 4, 0);
 	}
 
 	++nchild;
@@ -480,10 +494,10 @@ doit_active (char *host, char *user,
 	    for (i = 0; i < nsockets; ++i)
 		close (sockets[i].fd);
 
-	    addr = him;
+	    addr = kc->thataddr;
 	    close (otherside);
 
-	    addr.sin_port = htons(tmp);
+	    addr.sin_port = htons(other_port);
 	    s = socket (AF_INET, SOCK_STREAM, 0);
 	    if (s < 0)
 		err(1, "socket");
@@ -496,7 +510,7 @@ doit_active (char *host, char *user,
 	    }
 #endif
 #if defined(SO_KEEPALIVE) && defined(HAVE_SETSOCKOPT)
-	    if (keepalivep) {
+	    if (kc->keepalive_flag) {
 		int one = 1;
 
 		setsockopt (s, SOL_SOCKET, SO_KEEPALIVE, (void *)&one,
@@ -507,7 +521,7 @@ doit_active (char *host, char *user,
 	    if (connect (s, (struct sockaddr *)&addr, sizeof(addr)) < 0)
 		err(1, "connect");
 
-	    return active_session (fd, s, &key, schedule);
+	    return active_session (fd, s, kc);
 	} else {
 	    close (fd);
 	}
@@ -515,7 +529,7 @@ doit_active (char *host, char *user,
 }
 
 /*
- *
+ * Should we interpret `disp' as this being a passive call?
  */
 
 static int
@@ -529,82 +543,216 @@ check_for_passive (const char *disp)
 	(*disp == ':'
 	 || strncmp(disp, "unix", 4) == 0
 	 || strncmp(disp, "localhost", 9) == 0
-	 || strncmp(disp, local_hostname, strlen(local_hostname) == 0));
-}
-
-static void
-usage(void)
-{
-    fprintf(stderr, "Usage: %s [-p port] [-d] [-D] [-t] [-l remoteuser] host\n",
-	    __progname);
-    exit (1);
+	 || strncmp(disp, local_hostname, strlen(local_hostname)) == 0);
 }
 
 /*
- * kx - forward x connection over a kerberos-encrypted channel.
- *
+ * Set up signal handlers and then call the functions.
+ */
+
+static int
+doit (kx_context *kc, int passive_flag)
+{
+    signal (SIGCHLD, childhandler);
+    signal (SIGUSR1, usr1handler);
+    signal (SIGUSR2, usr2handler);
+    if (passive_flag)
+	return doit_passive (kc);
+    else
+	return doit_active  (kc);
+}
+
+#ifdef KRB4
+
+/*
+ * Start a v4-authenticatated kx connection.
+ */
+
+static int
+doit_v4 (const char *host, int port, const char *user, 
+	 int passive_flag, int debug_flag, int keepalive_flag, int tcp_flag)
+{
+    int ret;
+    kx_context context;
+
+    krb4_make_context (&context);
+    context_set (&context,
+		 host, user, port, debug_flag, keepalive_flag, tcp_flag);
+
+    ret = doit (&context, passive_flag);
+    context_destroy (&context);
+    return ret;
+}
+#endif /* KRB4 */
+
+#ifdef KRB5
+
+/*
+ * Start a v5-authenticatated kx connection.
+ */
+
+static int
+doit_v5 (const char *host, int port, const char *user,
+	 int passive_flag, int debug_flag, int keepalive_flag, int tcp_flag)
+{
+    int ret;
+    kx_context context;
+
+    krb5_make_context (&context);
+    context_set (&context,
+		 host, user, port, debug_flag, keepalive_flag, tcp_flag);
+
+    ret = doit (&context, passive_flag);
+    context_destroy (&context);
+    return ret;
+}
+#endif /* KRB5 */
+
+/*
+ * Variables set from the arguments
+ */
+
+#ifdef KRB4
+static int use_v4		= -1;
+#ifdef HAVE_KRB_ENABLE_DEBUG
+static int krb_debug_flag	= 0;
+#endif /* HAVE_KRB_ENABLE_DEBUG */
+#endif /* KRB4 */
+#ifdef KRB5
+static int use_v5		= -1;
+#endif
+static char *port_str		= NULL;
+static const char *user		= NULL;
+static int tcp_flag		= 0;
+static int passive_flag		= 0;
+static int keepalive_flag	= 1;
+static int debug_flag		= 0;
+static int version_flag		= 0;
+static int help_flag		= 0;
+
+struct getargs args[] = {
+#ifdef KRB4
+    { "krb4",	'4', arg_flag,		&use_v4,	"Use Kerberos V4",
+      NULL },
+#ifdef HAVE_KRB_ENABLE_DEBUG
+    { "krb4-debug", 'D', arg_flag,	&krb_debug_flag,
+      "enable krb4 debugging" },
+#endif /* HAVE_KRB_ENABLE_DEBUG */
+#endif /* KRB4 */
+#ifdef KRB5
+    { "krb5",	'5', arg_flag,		&use_v5,	"Use Kerberos V5",
+      NULL },
+#endif
+    { "port",	'p', arg_string,	&port_str,	"Use this port",
+      "number-of-service" },
+    { "user",	'l', arg_string,	&user,		"Run as this user",
+      NULL },
+    { "tcp",	't', arg_flag,		&tcp_flag,
+      "Use a TCP connection for X11" },
+    { "passive", 'P', arg_flag,		&passive_flag,
+      "Force a passive connection" },
+    { "keepalive", 'k', arg_negative_flag, &keepalive_flag,
+      "disable keep-alives" },
+    { "debug",	'd',	arg_flag,	&debug_flag,
+      "Enable debug information" },
+    { "version", 0,  arg_flag,		&version_flag,	"Print version",
+      NULL },
+    { "help",	 0,  arg_flag,		&help_flag,	NULL,
+      NULL }
+};
+
+static void
+usage(int ret)
+{
+    arg_printusage (args,
+		    sizeof(args) / sizeof(args[0]),
+		    NULL,
+		    "host");
+    exit (ret);
+}
+
+/*
+ * kx - forward an x-connection over a kerberos-encrypted channel.
  */
 
 int
 main(int argc, char **argv)
 {
-     int force_passive = 0;
-     int keepalivep = 1;
-     char *user = NULL;
-     int debugp = 0, tcpp = 0;
-     int c;
-     int port = 0;
+    int port	= 0;
+    int optind	= 0;
+    int ret	= 1;
+    char *host	= NULL;
 
-     set_progname (argv[0]);
-     while((c = getopt(argc, argv, "ktdDl:p:P")) != EOF) {
-	 switch(c) {
-	 case 'd' :
-	     debugp = 1;
-	     break;
-	 case 'D':
-	     krb_enable_debug();
-	     break;
-	 case 'k':
-	     keepalivep = 0;
-	     break;
-	 case 't' :
-	     tcpp = 1;
-	     break;
-	 case 'l' :
-	     user = optarg;
-	     break;
-	 case 'p' :
-	     port = htons(atoi (optarg));
-	     break;
-	 case 'P' :
-	     force_passive = 1;
-	     break;
-	 case '?':
-	 default:
-	     usage();
-	 }
-     }
+    set_progname (argv[0]);
 
-     argc -= optind;
-     argv += optind;
+    if (getarg (args, sizeof(args) / sizeof(args[0]), argc, argv,
+		&optind))
+	usage (1);
 
-     if (argc != 1)
-	  usage ();
-     if (user == NULL) {
-	  struct passwd *p = k_getpwuid (getuid ());
-	  if (p == NULL)
-	      errx(1, "Who are you?");
-	  user = strdup (p->pw_name);
-	  if (user == NULL)
-	      errx (1, "strdup: out of memory");
-     }
-     if (port == 0)
-	 port = k_getportbyname ("kx", "tcp", htons(KX_PORT));
-     signal (SIGCHLD, childhandler);
-     signal (SIGUSR1, usr1handler);
-     signal (SIGUSR2, usr2handler);
-     if (check_for_passive(getenv("DISPLAY")))
-	 return doit_passive (argv[0], user, debugp, keepalivep, port);
-     else
-	 return doit_active  (argv[0], user, debugp, keepalivep, tcpp, port);
+    if (help_flag)
+	usage (0);
+
+    if (version_flag) {
+	print_version (NULL);
+	return 0;
+    }
+
+    if (optind != argc - 1)
+	usage (1);
+
+    host = argv[optind];
+
+    if (port_str) {
+	struct servent *s = roken_getservbyname (port_str, "tcp");
+
+	if (s)
+	    port = s->s_port;
+	else {
+	    char *ptr;
+
+	    port = strtol (port_str, &ptr, 10);
+	    if (port == 0 && ptr == port_str)
+		errx (1, "Bad port `%s'", port_str);
+	    port = htons(port);
+	}
+    }
+
+    if (user == NULL) {
+	user = get_default_username ();
+	if (user == NULL)
+	    errx (1, "who are you?");
+    }
+
+    if (!passive_flag)
+	passive_flag = check_for_passive (getenv("DISPLAY"));
+
+#if defined(HAVE_KERNEL_ENABLE_DEBUG)
+    if (krb_debug_flag)
+	krb_enable_debug ();
+#endif
+
+#if defined(KRB4) && defined(KRB5)
+    if(use_v4 == -1 && use_v5 == 1)
+	use_v4 = 0;
+    if(use_v5 == -1 && use_v4 == 1)
+	use_v5 = 0;
+#endif    
+
+#ifdef KRB5
+    if (ret && use_v5) {
+	if (port == 0)
+	    port = krb5_getportbyname(NULL, "kx", "tcp", KX_PORT);
+	ret = doit_v5 (host, port, user,
+		       passive_flag, debug_flag, keepalive_flag, tcp_flag);
+    }
+#endif
+#ifdef KRB4
+    if (ret && use_v4) {
+	if (port == 0)
+	    port = k_getportbyname("kx", "tcp", htons(KX_PORT));
+	ret = doit_v4 (host, port, user, 
+		       passive_flag, debug_flag, keepalive_flag, tcp_flag);
+    }
+#endif
+    return ret;
 }
