@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #include "kx.h"
 
-RCSID("$Id: kxd.c,v 1.56.2.2 1999/09/02 19:12:09 assar Exp $");
+RCSID("$Id: kxd.c,v 1.61 1999/12/02 16:58:32 joda Exp $");
 
 static pid_t wait_on_pid = -1;
 static int   done        = 0;
@@ -160,12 +155,14 @@ recv_conn (int sock, kx_context *kc,
      if (ret) {
 	 syslog (LOG_ERR, "unrecognized auth protocol: %x %x %x %x",
 		 msg[0], msg[1], msg[2], msg[3]);
-	 return 1;
+	 exit (1);
      }
 
      len = kx_read (kc, sock, msg, sizeof(msg));
-     if (len < 0)
-	 return 1;
+     if (len < 0) {
+	 syslog (LOG_ERR, "kx_read failed");
+	 exit (1);
+     }
      p = (u_char *)msg;
      if (*p != INIT)
 	 fatal(kc, sock, "Bad message");
