@@ -11,7 +11,7 @@
 
 #include "sample.h"
 
-RCSID("$Id: simple_server.c,v 1.8 1997/03/30 17:38:25 assar Exp $");
+RCSID("$Id: simple_server.c,v 1.11 1999/11/13 06:29:24 assar Exp $");
 
 static void
 usage (void)
@@ -40,12 +40,12 @@ main(int argc, char **argv)
     des_key_schedule sched;
 
     set_progname (argv[0]);
-    strcpy (service, SAMPLE_SERVICE);
-    strcpy (instance, "*");
+    strlcpy (service, SAMPLE_SERVICE, sizeof(service));
+    strlcpy (instance, "*", sizeof(instance));
     *srvtab = '\0';
     port = 0;
 
-    while ((c = getopt (argc, argv, "p:s:t:")) != EOF)
+    while ((c = getopt (argc, argv, "p:s:t:")) != -1)
 	switch (c) {
 	case 'p' : {
 	    struct servent *sp;
@@ -58,12 +58,10 @@ main(int argc, char **argv)
 	    break;
 	}
 	case 's' :
-	    strncpy (service, optarg, sizeof(service));
-	    service[sizeof(service) - 1 ] = '\0';
+	    strlcpy (service, optarg, sizeof(service));
 	    break;
 	case 't' :
-	    strncpy (srvtab, optarg, sizeof(srvtab));
-	    srvtab[sizeof(srvtab) - 1 ] = '\0';
+	    strlcpy (srvtab, optarg, sizeof(srvtab));
 	    break;
 	case '?' :
 	default :
