@@ -33,7 +33,7 @@
 
 #include "telnet_locl.h"
 
-RCSID("$Id: sys_bsd.c,v 1.22 1998/05/27 02:19:14 assar Exp $");
+RCSID("$Id: sys_bsd.c,v 1.23 1998/06/09 19:24:46 joda Exp $");
 
 /*
  * The following routines try to encapsulate what is system dependent
@@ -713,7 +713,7 @@ ayt(int sig)
     if (connected)
 	sendayt();
     else
-	ayt_status();
+	ayt_status(sig);
 }
 #endif
 
