@@ -31,35 +31,14 @@
  * SUCH DAMAGE.
  */
 
-#include <config.h>
-#ifdef SOCKS
-#include <socks.h>
+#include "telnet_locl.h"
+#ifdef HAVE_TERMCAP_H
+#include <termcap.h>
 #endif
 
-RCSID("$Id: telnet.c,v 1.13 1996/11/17 20:23:28 assar Exp $");
+RCSID("$Id: telnet.c,v 1.25 1999/03/11 13:49:34 joda Exp $");
 
-#include <sys/types.h>
-#include <stdlib.h>
-
-#include <signal.h>
-/* By the way, we need to include curses.h before telnet.h since,
- * among other things, telnet.h #defines 'DO', which is a variable
- * declared in curses.h.
- */
-
-#include <arpa/telnet.h>
-
-#include <ctype.h>
-
-#include "ring.h"
-
-#include "defines.h"
-#include "externs.h"
-#include "types.h"
-#include "general.h"
-
-
-#define	strip(x) ((my_want_state_is_wont(TELOPT_BINARY)) ? ((x)&0x7f) : (x))
+#define	strip(x) (eight ? (x) : ((x) & 0x7f))
 
 static unsigned char	subbuffer[SUBBUFSIZE],
 			*subpointer, *subend;	 /* buffer for sub-options */
@@ -80,6 +59,7 @@ char	will_wont_resp[256];
 
 int
 	eight = 3,
+	binary = 0,
 	autologin = 0,	/* Autologin anyone? */
 	skiprc = 0,
 	connected,
@@ -130,7 +110,7 @@ unsigned char telopt_environ = TELOPT_NEW_ENVIRON;
 # define telopt_environ TELOPT_NEW_ENVIRON
 #endif
 
-jmp_buf	toplevel = { 0 };
+jmp_buf	toplevel;
 jmp_buf	peerdied;
 
 int	flushline;
@@ -147,17 +127,20 @@ int	kludgelinemode = 1;
 
 Clocks clocks;
 
+static int is_unique(char *name, char **as, char **ae);
+
+
 /*
  * Initialize telnet environment.
  */
 
 void
-init_telnet()
+init_telnet(void)
 {
     env_init();
 
     SB_CLEAR();
-    ClearArray(options);
+    memset(options, 0, sizeof options);
 
     connected = ISend = localflow = donebinarytoggle = 0;
 #if	defined(AUTHENTICATION) || defined(ENCRYPTION)
@@ -525,7 +508,7 @@ dontoption(int option)
 static char *name_unknown = "UNKNOWN";
 static char *unknown[] = { 0, 0 };
 
-char **
+static char **
 mklist(char *buf, char *name)
 {
 	int n;
@@ -537,7 +520,7 @@ mklist(char *buf, char *name)
 			unknown[0] = name_unknown;
 		} else {
 			unknown[0] = name;
-			upcase(name);
+			strupr(name);
 		}
 	} else
 		unknown[0] = name_unknown;
@@ -636,7 +619,7 @@ mklist(char *buf, char *name)
 		return(unknown);
 }
 
-int
+static int
 is_unique(char *name, char **as, char **ae)
 {
 	char **ap;
@@ -649,11 +632,10 @@ is_unique(char *name, char **as, char **ae)
 	return (1);
 }
 
-#ifndef HAVE_SETUPTERM
-char termbuf[1024];
+static char termbuf[1024];
 
-int
-setupterm(char *tname, int fd, int *errp)
+static int
+telnet_setupterm(const char *tname, int fd, int *errp)
 {
 	if (tgetent(termbuf, tname) == 1) {
 		termbuf[1023] = '\0';
@@ -665,14 +647,10 @@ setupterm(char *tname, int fd, int *errp)
 		*errp = 0;
 	return(-1);
 }
-#else
-#define	termbuf	ttytype
-extern char ttytype[];
-#endif
 
 int resettermname = 1;
 
-char *
+static char *
 gettermname()
 {
 	char *tname;
@@ -685,12 +663,12 @@ gettermname()
 		if (tnamep && tnamep != unknown)
 			free(tnamep);
 		if ((tname = (char *)env_getvalue((unsigned char *)"TERM")) &&
-				(setupterm(tname, 1, &err) == 0)) {
+				telnet_setupterm(tname, 1, &err) == 0) {
 			tnamep = mklist(termbuf, tname);
 		} else {
 			if (tname && ((int)strlen(tname) <= 40)) {
 				unknown[0] = tname;
-				upcase(tname);
+				strupr(tname);
 			} else
 				unknown[0] = name_unknown;
 			tnamep = unknown;
@@ -735,8 +713,9 @@ suboption()
 	    name = gettermname();
 	    len = strlen(name) + 4 + 2;
 	    if (len < NETROOM()) {
-		sprintf((char *)temp, "%c%c%c%c%s%c%c", IAC, SB, TELOPT_TTYPE,
-				TELQUAL_IS, name, IAC, SE);
+		snprintf((char *)temp, sizeof(temp),
+			 "%c%c%c%c%s%c%c", IAC, SB, TELOPT_TTYPE,
+			 TELQUAL_IS, name, IAC, SE);
 		ring_supply_data(&netoring, temp, len);
 		printsub('>', &temp[2], len-2);
 	    } else {
@@ -751,14 +730,17 @@ suboption()
 	if (SB_EOF())
 	    return;
 	if (SB_GET() == TELQUAL_SEND) {
-	    long ospeed, ispeed;
+	    long output_speed, input_speed;
 	    unsigned char temp[50];
 	    int len;
 
-	    TerminalSpeeds(&ispeed, &ospeed);
+	    TerminalSpeeds(&input_speed, &output_speed);
 
-	    sprintf((char *)temp, "%c%c%c%c%d,%d%c%c", IAC, SB, TELOPT_TSPEED,
-		    TELQUAL_IS, ospeed, ispeed, IAC, SE);
+	    snprintf((char *)temp, sizeof(temp),
+		     "%c%c%c%c%u,%u%c%c", IAC, SB, TELOPT_TSPEED,
+		     TELQUAL_IS,
+		     (unsigned)output_speed,
+		     (unsigned)input_speed, IAC, SE);
 	    len = strlen((char *)temp+4) + 4;	/* temp[3] is 0 ... */
 
 	    if (len < NETROOM()) {
@@ -862,8 +844,9 @@ suboption()
 		send_wont(TELOPT_XDISPLOC, 1);
 		break;
 	    }
-	    sprintf((char *)temp, "%c%c%c%c%s%c%c", IAC, SB, TELOPT_XDISPLOC,
-		    TELQUAL_IS, dp, IAC, SE);
+	    snprintf((char *)temp, sizeof(temp),
+		     "%c%c%c%c%s%c%c", IAC, SB, TELOPT_XDISPLOC,
+		     TELQUAL_IS, dp, IAC, SE);
 	    len = strlen((char *)temp+4) + 4;	/* temp[3] is 0 ... */
 
 	    if (len < NETROOM()) {
@@ -1103,7 +1086,7 @@ slc_init()
 
 #define	initfunc(func, flags) { \
 					spcp = &spc_data[func]; \
-					if (spcp->valp = tcval(func)) { \
+					if ((spcp->valp = tcval(func))) { \
 					    spcp->val = *spcp->valp; \
 					    spcp->mylevel = SLC_VARIABLE|flags; \
 					} else { \
@@ -1467,12 +1450,12 @@ env_opt_add(unsigned char *ep)
 	if (ep == NULL || *ep == '\0') {
 		/* Send user defined variables first. */
 		env_default(1, 0);
-		while (ep = env_default(0, 0))
+		while ((ep = env_default(0, 0)))
 			env_opt_add(ep);
 
 		/* Now add the list of well know variables.  */
 		env_default(1, 1);
-		while (ep = env_default(0, 1))
+		while ((ep = env_default(0, 1)))
 			env_opt_add(ep);
 		return;
 	}
@@ -1492,17 +1475,17 @@ env_opt_add(unsigned char *ep)
 		opt_replyp = opt_reply + len - (opt_replyend - opt_replyp);
 		opt_replyend = opt_reply + len;
 	}
-	if (opt_welldefined(ep))
+	if (opt_welldefined((char *)ep)) {
 #ifdef	OLD_ENVIRON
 		if (telopt_environ == TELOPT_OLD_ENVIRON)
 			*opt_replyp++ = old_env_var;
 		else
 #endif
 			*opt_replyp++ = NEW_ENV_VAR;
-	else
+	} else
 		*opt_replyp++ = ENV_USERVAR;
 	for (;;) {
-		while (c = *ep++) {
+		while ((c = *ep++)) {
 			switch(c&0xff) {
 			case IAC:
 				*opt_replyp++ = IAC;
@@ -1516,7 +1499,7 @@ env_opt_add(unsigned char *ep)
 			}
 			*opt_replyp++ = c;
 		}
-		if (ep = vp) {
+		if ((ep = vp)) {
 #ifdef	OLD_ENVIRON
 			if (telopt_environ == TELOPT_OLD_ENVIRON)
 				*opt_replyp++ = old_env_value;
@@ -1566,11 +1549,11 @@ env_opt_end(int emptyok)
 
 
 int
-telrcv()
+telrcv(void)
 {
     int c;
     int scc;
-    unsigned char *sbp;
+    unsigned char *sbp = NULL;
     int count;
     int returnValue = 0;
 
@@ -1615,9 +1598,9 @@ telrcv()
 		telrcv_state = TS_IAC;
 		break;
 	    }
-		    /*
+		    /* 
 		     * The 'crmod' hack (see following) is needed
-		     * since we can't * set CRMOD on output only.
+		     * since we can't set CRMOD on output only.
 		     * Machines like MULTICS like to send \r without
 		     * \n; since we must turn off CRMOD to get proper
 		     * input, the mapping is done here (sigh).
@@ -1798,7 +1781,7 @@ process_iac:
 static int bol = 1, local = 0;
 
 int
-rlogin_susp()
+rlogin_susp(void)
 {
     if (local) {
 	local = 0;
@@ -1815,7 +1798,7 @@ telsnd()
     int tcc;
     int count;
     int returnValue = 0;
-    unsigned char *tbp;
+    unsigned char *tbp = NULL;
 
     tcc = 0;
     count = 0;
@@ -1960,7 +1943,7 @@ telsnd()
  */
 
 
-int
+static int
 Scheduler(int block) /* should we block in the select ? */
 {
 		/* One wants to be a bit careful about setting returnValue
@@ -2011,7 +1994,7 @@ Scheduler(int block) /* should we block in the select ? */
  * Select from tty and network...
  */
 void
-telnet(char *user)
+my_telnet(char *user)
 {
     sys_telnet_init();
 
@@ -2020,7 +2003,8 @@ telnet(char *user)
 	static char local_host[256] = { 0 };
 
 	if (!local_host[0]) {
-		k_gethostname(local_host, sizeof(local_host));
+		/* XXX - should be k_gethostname? */
+		gethostname(local_host, sizeof(local_host));
 		local_host[sizeof(local_host)-1] = 0;
 	}
 	auth_encrypt_init(local_host, hostname, "TELNET", 0);
@@ -2046,8 +2030,8 @@ telnet(char *user)
 	send_do(TELOPT_STATUS, 1);
 	if (env_getvalue((unsigned char *)"DISPLAY"))
 	    send_will(TELOPT_XDISPLOC, 1);
-	if (eight)
-	    tel_enter_binary(eight);
+	if (binary)
+	    tel_enter_binary(binary);
     }
 
     for (;;) {
@@ -2140,7 +2124,7 @@ doflush()
 }
 
 void
-xmitAO()
+xmitAO(void)
 {
     NET2ADD(IAC, AO);
     printoption("SENT", IAC, AO);
@@ -2151,14 +2135,14 @@ xmitAO()
 
 
 void
-xmitEL()
+xmitEL(void)
 {
     NET2ADD(IAC, EL);
     printoption("SENT", IAC, EL);
 }
 
 void
-xmitEC()
+xmitEC(void)
 {
     NET2ADD(IAC, EC);
     printoption("SENT", IAC, EC);
@@ -2205,7 +2189,7 @@ get_status()
 }
 
 void
-intp()
+intp(void)
 {
     NET2ADD(IAC, IP);
     printoption("SENT", IAC, IP);
@@ -2219,7 +2203,7 @@ intp()
 }
 
 void
-sendbrk()
+sendbrk(void)
 {
     NET2ADD(IAC, BREAK);
     printoption("SENT", IAC, BREAK);
@@ -2233,7 +2217,7 @@ sendbrk()
 }
 
 void
-sendabort()
+sendabort(void)
 {
     NET2ADD(IAC, ABORT);
     printoption("SENT", IAC, ABORT);
@@ -2247,7 +2231,7 @@ sendabort()
 }
 
 void
-sendsusp()
+sendsusp(void)
 {
     NET2ADD(IAC, SUSP);
     printoption("SENT", IAC, SUSP);
@@ -2261,14 +2245,14 @@ sendsusp()
 }
 
 void
-sendeof()
+sendeof(void)
 {
     NET2ADD(IAC, xEOF);
     printoption("SENT", IAC, xEOF);
 }
 
 void
-sendayt()
+sendayt(void)
 {
     NET2ADD(IAC, AYT);
     printoption("SENT", IAC, AYT);
