@@ -31,57 +31,17 @@
  * SUCH DAMAGE.
  */
 
-#include <config.h>
-#ifdef SOCKS
-#include <socks.h>
-#endif
+#define TELOPTS
+#define TELCMDS
+#define SLC_NAMES
 
-RCSID("$Id: utilities.c,v 1.12 1996/11/17 20:23:28 assar Exp $");
+#include "telnet_locl.h"
 
-#define	TELOPTS
-#define	TELCMDS
-#define	SLC_NAMES
-#include <arpa/telnet.h>
-#include <sys/types.h>
-#ifdef HAVE_SYS_SELECT_H
-#include <sys/select.h>
-#endif
-#include <sys/time.h>
-
-#include <ctype.h>
-
-#include "general.h"
-
-#include "fdset.h"
-
-#include "ring.h"
-
-#include "defines.h"
-
-#include "externs.h"
+RCSID("$Id: utilities.c,v 1.22 1999/09/16 20:41:36 assar Exp $");
 
 FILE	*NetTrace = 0;		/* Not in bss, since needs to stay */
 int	prettydump;
 
-/*
- * upcase()
- *
- *	Upcase (in place) the argument.
- */
-
-void
-upcase(char *argument)
-{
-    int c;
-
-    while ((c = *argument) != 0) {
-	if (islower(c)) {
-	    *argument = toupper(c);
-	}
-	argument++;
-    }
-}
-
 /*
  * SetSockOpt()
  *
@@ -91,6 +51,7 @@ upcase(char *argument)
 int
 SetSockOpt(int fd, int level, int option, int yesno)
 {
+#ifdef HAVE_SETSOCKOPT
 #ifndef	NOT43
     return setsockopt(fd, level, option,
 				(void *)&yesno, sizeof yesno);
@@ -102,13 +63,16 @@ SetSockOpt(int fd, int level, int option, int yesno)
     }
     return setsockopt(fd, level, option, 0, 0);
 #endif	/* NOT43 */
+#else
+    return -1;
+#endif
 }
 
 /*
  * The following are routines used to print out debugging information.
  */
 
-unsigned char NetTraceFile[256] = "(standard output)";
+char NetTraceFile[256] = "(standard output)";
 
 void
 SetNetTrace(char *file)
@@ -118,23 +82,21 @@ SetNetTrace(char *file)
     if (file  && (strcmp(file, "-") != 0)) {
 	NetTrace = fopen(file, "w");
 	if (NetTrace) {
-	    strcpy((char *)NetTraceFile, file);
+	    strlcpy(NetTraceFile, file, sizeof(NetTraceFile));
 	    return;
 	}
 	fprintf(stderr, "Cannot open %s.\n", file);
     }
     NetTrace = stdout;
-    strcpy((char *)NetTraceFile, "(standard output)");
+    strlcpy(NetTraceFile, "(standard output)", sizeof(NetTraceFile));
 }
 
 void
 Dump(char direction, unsigned char *buffer, int length)
 {
 #   define BYTES_PER_LINE	32
-#   define min(x,y)	((x<y)? x:y)
     unsigned char *pThis;
     int offset;
-    extern int pettydump;
 
     offset = 0;
 
@@ -288,10 +250,10 @@ optionstatus(void)
 }
 
 void
-printsub(char direction, unsigned char *pointer, int length)
+printsub(int direction, unsigned char *pointer, int length)
 {
     int i;
-    char buf[512];
+    unsigned char buf[512];
     extern int want_status_response;
 
     if (showoptions || direction == 0 ||
@@ -630,12 +592,13 @@ printsub(char direction, unsigned char *pointer, int length)
 		}
 		{
 		    char tbuf[64];
-		    sprintf(tbuf, "%s%s%s%s%s",
-			pointer[2]&MODE_EDIT ? "|EDIT" : "",
-			pointer[2]&MODE_TRAPSIG ? "|TRAPSIG" : "",
-			pointer[2]&MODE_SOFT_TAB ? "|SOFT_TAB" : "",
-			pointer[2]&MODE_LIT_ECHO ? "|LIT_ECHO" : "",
-			pointer[2]&MODE_ACK ? "|ACK" : "");
+		    snprintf(tbuf, sizeof(tbuf),
+			     "%s%s%s%s%s",
+			     pointer[2]&MODE_EDIT ? "|EDIT" : "",
+			     pointer[2]&MODE_TRAPSIG ? "|TRAPSIG" : "",
+			     pointer[2]&MODE_SOFT_TAB ? "|SOFT_TAB" : "",
+			     pointer[2]&MODE_LIT_ECHO ? "|LIT_ECHO" : "",
+			     pointer[2]&MODE_ACK ? "|ACK" : "");
 		    fprintf(NetTrace, "%s", tbuf[1] ? &tbuf[1] : "0");
 		}
 		if (pointer[2]&~(MODE_MASK))
@@ -801,7 +764,6 @@ printsub(char direction, unsigned char *pointer, int length)
 			    break;
 
 			default:
-			def_case:
 			    if (isprint(pointer[i]) && pointer[i] != '"') {
 				if (noquote) {
 				    putc('"', NetTrace);
