@@ -36,8 +36,9 @@
 /*
  * Telnet server defines
  */
-#include <sys/types.h>
-#include <sys/param.h>
+
+#ifndef __DEFS_H__
+#define __DEFS_H__
 
 #ifndef	BSD
 # define	BSD 43
@@ -49,75 +50,6 @@
 #define	SLC_NAMES
 #endif
 
-#include <sys/socket.h>
-#include <sys/time.h>
-#ifdef HAVE_SYS_RESOURCE_H
-#include <sys/resource.h>
-#endif /* HAVE_SYS_RESOURCE_H */
-#ifndef	CRAY
-#include <sys/wait.h>
-#endif	/* CRAY */
-#include <fcntl.h>
-#include <sys/file.h>
-#include <sys/stat.h>
-
-/* including both <sys/ioctl.h> and <termios.h> in SunOS 4 generates a
-   lot of warnings */
-
-#if defined(HAVE_SYS_IOCTL_H) && SunOS != 4
-#include <sys/ioctl.h>
-#endif
-#ifdef HAVE_SYS_FILIO_H
-#include <sys/filio.h>
-#endif
-
-#include <netinet/in.h>
-#include <arpa/inet.h>
-
-#include <arpa/telnet.h>
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <signal.h>
-#include <errno.h>
-#include <netdb.h>
-#include <syslog.h>
-#ifndef	LOG_DAEMON
-#define	LOG_DAEMON	0
-#endif
-#ifndef	LOG_ODELAY
-#define	LOG_ODELAY	0
-#endif
-#include <ctype.h>
-#include <string.h>
-
-#include <termios.h>
-#include <unistd.h>
-
-#ifndef _POSIX_VDISABLE
-# ifdef VDISABLE
-#  define _POSIX_VDISABLE VDISABLE
-# else
-#  define _POSIX_VDISABLE ((unsigned char)'\377')
-# endif
-#endif
-
-
-#ifdef	CRAY
-# ifdef	CRAY1
-# include <sys/pty.h>
-#  ifndef FD_ZERO
-# include <sys/select.h>
-#  endif /* FD_ZERO */
-# endif	/* CRAY1 */
-
-#include <memory.h>
-#endif	/* CRAY */
-
-#ifdef __hpux
-#include <sys/ptyio.h>
-#endif
-
 #if	!defined(TIOCSCTTY) && defined(TCSETCTTY)
 # define	TIOCSCTTY TCSETCTTY
 #endif
@@ -134,22 +66,11 @@
 #define TIOCPKT_DOSTOP  0x20
 #endif
 
-#ifndef	FD_SET
-#ifndef	HAVE_fd_set
-typedef struct fd_set { int fds_bits[1]; } fd_set;
-#endif
-
-#define	FD_SET(n, p)	((p)->fds_bits[0] |= (1<<(n)))
-#define	FD_CLR(n, p)	((p)->fds_bits[0] &= ~(1<<(n)))
-#define	FD_ISSET(n, p)	((p)->fds_bits[0] & (1<<(n)))
-#define FD_ZERO(p)	((p)->fds_bits[0] = 0)
-#endif	/* FD_SET */
-
 /*
  * I/O data buffers defines
  */
 #define	NETSLOP	64
-#ifdef CRAY
+#ifdef _CRAY
 #undef BUFSIZ
 #define BUFSIZ  2048
 #endif
@@ -162,15 +83,6 @@ typedef struct fd_set { int fds_bits[1]; } fd_set;
 #define	settimer(x)	(clocks.x = ++clocks.system)
 #define	sequenceIs(x,y)	(clocks.x < clocks.y)
 
-/*
- * Linemode support states, in decreasing order of importance
- */
-#define REAL_LINEMODE	0x04
-#define KLUDGE_OK	0x03
-#define	NO_AUTOKLUDGE	0x02
-#define KLUDGE_LINEMODE	0x01
-#define NO_LINEMODE	0x00
-
 /*
  * Structures of information for each special character function.
  */
@@ -274,3 +186,5 @@ typedef struct {
 
 #define his_will_wont_is_changing	my_do_dont_is_changing
 #define his_do_dont_is_changing		my_will_wont_is_changing
+
+#endif /* __DEFS_H__ */
