@@ -33,7 +33,7 @@
 
 #include "telnetd.h"
 
-RCSID("$Id: sys_term.c,v 1.79 1997/10/10 22:07:24 assar Exp $");
+RCSID("$Id: sys_term.c,v 1.89 1999/09/16 20:41:36 assar Exp $");
 
 #if defined(_CRAY) || (defined(__hpux) && !defined(HAVE_UTMPX_H))
 # define PARENT_DOES_UTMP
@@ -53,7 +53,7 @@ struct	utmpx wtmp;
 struct	utmp wtmp;
 #endif /* HAVE_UTMPX_H */
 
-#ifdef HAVE_UT_HOST
+#ifdef HAVE_STRUCT_UTMP_UT_HOST
 int	utmp_len = sizeof(wtmp.ut_host);
 #else
 int	utmp_len = MaxHostNameLen;
@@ -360,7 +360,7 @@ char *line = Xline;
 char myline[] = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
 #endif	/* CRAY */
 
-#ifndef HAVE_PTSNAME
+#if !defined(HAVE_PTSNAME) && defined(STREAMSPTY)
 static char *ptsname(int fd)
 {
 #ifdef HAVE_TTYNAME
@@ -388,14 +388,14 @@ int getpty(int *ptynum)
     p = _getpty(&master, O_RDWR, 0600, 1);
     if(p == NULL)
 	return -1;
-    strcpy(line, p);
+    strlcpy(line, p, sizeof(Xline));
     return master;
 #else
 
     int p;
     char *cp, *p1, *p2;
     int i;
-#if SunOS == 4
+#if SunOS == 40
     int dummy;
 #endif
 #if 0 /* && defined(HAVE_OPENPTY) */
@@ -420,7 +420,7 @@ int getpty(int *ptynum)
 #ifdef HAVE_UNLOCKPT
 	    unlockpt(p);
 #endif
-	    strcpy(line, ptsname(p));
+	    strlcpy(line, ptsname(p), sizeof(Xline));
 	    really_stream = 1;
 	    return p;
 	}
@@ -464,14 +464,14 @@ int getpty(int *ptynum)
 #endif
 		chown(line, 0, 0);
 		chmod(line, 0600);
-#if SunOS == 4
+#if SunOS == 40
 		if (ioctl(p, TIOCGPGRP, &dummy) == 0
 		    || errno != EIO) {
 		    chmod(line, 0666);
 		    close(p);
 		    line[5] = 'p';
 		} else
-#endif /* SunOS == 4 */
+#endif /* SunOS == 40 */
 		    return(p);
 	    }
 	}
@@ -1087,6 +1087,7 @@ clean_ttyname (char *tty)
  * Generate a name usable as an `ut_id', typically without `tty'.
  */
 
+#ifdef HAVE_STRUCT_UTMP_UT_ID
 static char *
 make_id (char *tty)
 {
@@ -1098,6 +1099,7 @@ make_id (char *tty)
     res += 3;
   return res;
 }
+#endif
 
 /*
  * startslave(host)
@@ -1158,7 +1160,7 @@ startslave(char *host, int autologin, char *autoname)
 	strncpy(wtmp.ut_user,  "LOGIN", sizeof(wtmp.ut_user));
 	strncpy(wtmp.ut_host,  host, sizeof(wtmp.ut_host));
 	strncpy(wtmp.ut_line,  clean_ttyname(line), sizeof(wtmp.ut_line));
-#ifdef _CRAY /* XXX */
+#ifdef HAVE_STRUCT_UTMP_UT_ID
 	strncpy(wtmp.ut_id, wtmp.ut_line + 3, sizeof(wtmp.ut_id));
 #endif
 
@@ -1190,7 +1192,7 @@ init_env(void)
     char **envp;
 
     envp = envinit;
-    if (*envp = getenv("TZ"))
+    if ((*envp = getenv("TZ")))
 	*envp++ -= 3;
 #if defined(_CRAY) || defined(__hpux)
     else
@@ -1248,11 +1250,8 @@ start_login(char *host, int autologin, char *name)
     char *user;
 
 #ifdef HAVE_UTMPX_H
-    char id_buf[3];
-    int ptynum;
     int pid = getpid();
     struct utmpx utmpx;
-    struct timeval tmp;
     char *clean_tty;
 
     /*
@@ -1263,7 +1262,7 @@ start_login(char *host, int autologin, char *name)
     memset(&utmpx, 0, sizeof(utmpx));
     strncpy(utmpx.ut_user,  ".telnet", sizeof(utmpx.ut_user));
     strncpy(utmpx.ut_line,  clean_tty, sizeof(utmpx.ut_line));
-#ifdef HAVE_UT_ID
+#ifdef HAVE_STRUCT_UTMP_UT_ID
     strncpy(utmpx.ut_id, make_id(clean_tty), sizeof(utmpx.ut_id));
 #endif
     utmpx.ut_pid = pid;
@@ -1300,9 +1299,13 @@ start_login(char *host, int autologin, char *name)
 	user = getenv("USER");
 #ifdef AUTHENTICATION
     if (auth_level < 0 || autologin != AUTH_VALID) {
-	if(!no_warn)
-	    printf("User not authenticated. "
-		   "Using plaintext username and password\r\n");
+	if(!no_warn) {
+	    printf("User not authenticated. ");
+	    if (require_otp)
+		printf("Using one-time password\r\n");
+	    else
+		printf("Using plaintext username and password\r\n");
+	}
 	if (require_otp) {
 	    addarg(&argv, "-a");
 	    addarg(&argv, "otp");
@@ -1373,13 +1376,7 @@ static int addarg(struct arg_val *argv, char *val)
 static void
 rmut(void)
 {
-    int f;
-    int found = 0;
-    struct utmp *u, *utmp;
-    int nutmp;
-    struct stat statbf;
-    struct timeval tmp;
-    struct utmpx *utxp, utmpx;
+    struct utmpx utmpx, *non_save_utxp;
     char *clean_tty = clean_ttyname(line);
 
     /*
@@ -1390,11 +1387,17 @@ rmut(void)
     memset(&utmpx, 0, sizeof(utmpx));
     strncpy(utmpx.ut_line, clean_tty, sizeof(utmpx.ut_line));
     utmpx.ut_type = LOGIN_PROCESS;
-    utxp = getutxline(&utmpx);
-    if (utxp) {
-	strcpy(utxp->ut_user, "");
+    non_save_utxp = getutxline(&utmpx);
+    if (non_save_utxp) {
+	struct utmpx *utxp;
+	char user0;
+
+	utxp = malloc(sizeof(struct utmpx));
+	*utxp = *non_save_utxp;
+	user0 = utxp->ut_user[0];
+	utxp->ut_user[0] = '\0';
 	utxp->ut_type = DEAD_PROCESS;
-#ifdef HAVE_UT_EXIT
+#ifdef HAVE_STRUCT_UTMPX_UT_EXIT
 #ifdef _STRUCT___EXIT_STATUS
 	utxp->ut_exit.__e_termination = 0;
 	utxp->ut_exit.__e_exit = 0;
@@ -1409,6 +1412,7 @@ rmut(void)
 	gettimeofday(&utxp->ut_tv, NULL);
 	pututxline(utxp);
 #ifdef WTMPX_FILE
+	utxp->ut_user[0] = user0;
 	updwtmpx(WTMPX_FILE, utxp);
 #elif defined(WTMP_FILE)
 	/* This is a strange system with a utmpx and a wtmp! */
@@ -1418,7 +1422,7 @@ rmut(void)
 	  if (f >= 0) {
 	    strncpy(wtmp.ut_line,  clean_tty, sizeof(wtmp.ut_line));
 	    strncpy(wtmp.ut_name,  "", sizeof(wtmp.ut_name));
-#ifdef HAVE_UT_HOST
+#ifdef HAVE_STRUCT_UTMP_UT_HOST
 	    strncpy(wtmp.ut_host,  "", sizeof(wtmp.ut_host));
 #endif
 	    time(&wtmp.ut_time);
@@ -1426,9 +1430,8 @@ rmut(void)
 	    close(f);
 	  }
 	}
-#else
-
 #endif
+	free (utxp);
     }
     endutxent();
 }  /* end of rmut */
@@ -1463,7 +1466,7 @@ rmut(void)
 		    continue;
 		lseek(f, ((long)u)-((long)utmp), L_SET);
 		strncpy(u->ut_name,  "", sizeof(u->ut_name));
-#ifdef HAVE_UT_HOST
+#ifdef HAVE_STRUCT_UTMP_UT_HOST
 		strncpy(u->ut_host,  "", sizeof(u->ut_host));
 #endif
 		time(&u->ut_time);
@@ -1478,7 +1481,7 @@ rmut(void)
 	if (f >= 0) {
 	    strncpy(wtmp.ut_line,  clean_tty, sizeof(wtmp.ut_line));
 	    strncpy(wtmp.ut_name,  "", sizeof(wtmp.ut_name));
-#ifdef HAVE_UT_HOST
+#ifdef HAVE_STRUCT_UTMP_UT_HOST
 	    strncpy(wtmp.ut_host,  "", sizeof(wtmp.ut_host));
 #endif
 	    time(&wtmp.ut_time);
@@ -1600,8 +1603,10 @@ cleanup(int sig)
 #if defined(HAVE_UTMPX_H) || !defined(HAVE_LOGWTMP)
     rmut();
 #ifdef HAVE_VHANGUP
+#ifndef __sgi
     vhangup(); /* XXX */
 #endif
+#endif
 #else
     char *p;
     
