@@ -8,7 +8,7 @@
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: xnlock.c,v 1.71 1998/04/22 01:33:30 joda Exp $");
+RCSID("$Id: xnlock.c,v 1.78 1999/09/16 20:41:38 assar Exp $");
 #endif
 
 #include <stdio.h>
@@ -47,7 +47,7 @@ static char userprompt[128];
 #ifdef KRB4
 static char name[ANAME_SZ];
 static char inst[INST_SZ];
-static char realm[REALM_SZ + 1];
+static char realm[REALM_SZ];
 #endif
 #ifdef KRB5
 static krb5_context context;
@@ -251,8 +251,8 @@ init_words (int argc, char **argv)
 		errx (1, "cannot allocate memory for message");
 	    appres.text[0] = 0;
 	    for(; i < j; i++){
-		strcat(appres.text, argv[i]);
-		strcat(appres.text, " ");
+		strlcat(appres.text, argv[i], len);
+		strlcat(appres.text, " ", len);
 	    }
 	}
     }
@@ -632,7 +632,8 @@ verify(char *password)
     /*
      * Try to verify as user with kerberos 4.
      */
-    ret = krb_verify_user(name, inst, realm, password, 0, NULL);
+    ret = krb_verify_user(name, inst, realm, password,
+			  KRB_VERIFY_NOT_SECURE, NULL);
     if (ret == KSUCCESS){
 	if (k_hasafs())
 	    krb_afslog(NULL, NULL);
@@ -722,12 +723,12 @@ GetPasswd(Widget w, XEvent *_event, String *_s, Cardinal *_n)
 			 prompt_x + XTextWidth(font, STRING, echolen),
 			 prompt_y, SPACE_STRING, STRING_LENGTH - echolen + 1);
       }
-    } else if (isprint(c))
+    } else if (isprint(c)) {
 	if ((cnt + 1) >= MAX_PASSWD_LENGTH)
 	    XBell(dpy, 50);
 	else
 	    passwd[cnt++] = c;
-    else
+    } else
 	return;
     echolen = min(cnt, STRING_LENGTH);
     XDrawImageString(dpy, XtWindow(w), gc,
@@ -798,7 +799,8 @@ talk(int force_erase)
     XSetForeground(dpy, gc, White);
     talking = 1;
     walk(FRONT);
-    p = strcpy(buf, words);
+    strlcpy (buf, words, sizeof(buf));
+    p = buf;
 
     /* possibly avoid a lot of work here
      * if no CR or only one, then just print the line
@@ -832,7 +834,7 @@ talk(int force_erase)
 	if ((w = XTextWidth(font, p, p2 - p)) > width)
 	    width = w;
 	total += p2 - p; /* total chars; count to determine reading time */
-	strcpy(args[height], p);
+	strlcpy(args[height], p, sizeof(args[height]));
 	if (height == MAXLINES - 1) {
 	    puts("Message too long!");
 	    break;
@@ -927,15 +929,14 @@ main (int argc, char **argv)
       struct passwd *pw;
       if (!(pw = k_getpwuid(0)))
 	errx (1, "can't get root's passwd!");
-      strcpy(root_cpass, pw->pw_passwd);
+      strlcpy(root_cpass, pw->pw_passwd, sizeof(root_cpass));
 
       if (!(pw = k_getpwuid(getuid())))
 	errx (1, "Can't get your password entry!");
-      strcpy(user_cpass, pw->pw_passwd);
+      strlcpy(user_cpass, pw->pw_passwd, sizeof(user_cpass));
       setuid(getuid());
       /* Now we're no longer running setuid root. */
-      strncpy(login, pw->pw_name, sizeof(login));
-      login[sizeof(login) - 1] = '\0';
+      strlcpy(login, pw->pw_name, sizeof(login));
     }
 
     srand(getpid());
@@ -975,8 +976,13 @@ main (int argc, char **argv)
 
     if (appres.destroytickets) {
 #ifdef KRB4
+	int fd;
+
         dest_tkt();		/* Nuke old ticket file */
-	creat(TKT_FILE, 0600);	/* but keep a place holder */
+				/* but keep a place holder */
+	fd = open (TKT_FILE, O_WRONLY | O_CREAT | O_EXCL, 0600);
+	if (fd >= 0)
+	    close (fd);
 #endif
     }
 
