@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -30,7 +25,7 @@ or implied warranty.
 
 #include "kadm_locl.h"
 
-RCSID("$Id: kpasswd.c,v 1.21 1996/11/19 04:06:58 assar Exp $");
+RCSID("$Id: kpasswd.c,v 1.29 1999/11/13 06:33:20 assar Exp $");
 
 static void
 usage(int value)
@@ -48,12 +43,13 @@ main(int argc, char **argv)
     krb_principal default_principal;
     int realm_given = 0;	/* True if realm was give on cmdline */
     int use_default = 1;	/* True if we should use default name */
-    struct passwd *pw;
     int status;			/* return code */
     char pword[MAX_KPW_LEN];
     int c;
     char tktstring[MaxPathLen];
     
+    set_progname (argv[0]);
+
     memset (&principal, 0, sizeof(principal));
     memset (&default_principal, 0, sizeof(default_principal));
     
@@ -61,46 +57,45 @@ main(int argc, char **argv)
 			       default_principal.instance,
 			       default_principal.realm);
 
-    while ((c = getopt(argc, argv, "u:n:i:r:h")) != EOF) {
+    while ((c = getopt(argc, argv, "u:n:i:r:h")) != -1) {
 	switch (c) {
 	case 'u':
 	    status = krb_parse_name (optarg, &principal);
-	    if (status != KSUCCESS) {
-		fprintf (stderr, "Kerberos error: %s\n",
-			 krb_get_err_text(status));
-		exit (2);
-	    }
+	    if (status != KSUCCESS)
+		errx (2, "%s", krb_get_err_text(status));
 	    if (principal.realm[0])
 		realm_given++;
-	    else if (krb_get_lrealm(principal.realm, 1) != KSUCCESS) {
-		fprintf (stderr, "Could not find default realm!\n");
-		exit (1);
-	    }
+	    else if (krb_get_lrealm(principal.realm, 1) != KSUCCESS)
+		errx (1, "Could not find default realm!");
 	    break;
 	case 'n':
 	    if (k_isname(optarg))
-		strncpy(principal.name, optarg, sizeof(principal.name) - 1);
+		strlcpy(principal.name,
+				optarg,
+				sizeof(principal.name));
 	    else {
-		fprintf(stderr, "Bad name: %s\n", optarg);
+		warnx("Bad name: %s", optarg);
 		usage(1);
 	    }
 	    break;
 	case 'i':
 	    if (k_isinst(optarg))
-		strncpy(principal.instance,
-			optarg,
-			sizeof(principal.instance) - 1);
+		strlcpy(principal.instance,
+				optarg,
+				sizeof(principal.instance));
 	    else {
-		fprintf(stderr, "Bad instance: %s\n", optarg);
+		warnx("Bad instance: %s", optarg);
 		usage(1);
 	    }
 	    break;
 	case 'r':
 	    if (k_isrealm(optarg)) {
-		strncpy(principal.realm, optarg, sizeof(principal.realm) - 1);
+		strlcpy(principal.realm,
+				optarg,
+				sizeof(principal.realm));
 		realm_given++; 
 	    } else {
-		fprintf(stderr, "Bad realm: %s\n", optarg);
+		warnx("Bad realm: %s", optarg);
 		usage(1);
 	    }
 	    break;
@@ -116,24 +111,33 @@ main(int argc, char **argv)
     if (optind < argc) {
 	use_default = 0;
 	status = krb_parse_name (argv[optind], &principal);
-	if(status != KSUCCESS){
-	    fprintf(stderr, "Kerberos error: %s\n", krb_get_err_text(status));
-	    exit(1);
-	}
+	if(status != KSUCCESS)
+	    errx (1, "%s", krb_get_err_text (status));
     }
 
     if (use_default) {
-	strcpy(principal.name, default_principal.name);
-	strcpy(principal.instance, default_principal.instance);
-	strcpy(principal.realm, default_principal.realm);
+	strlcpy(principal.name,
+			default_principal.name,
+			sizeof(principal.name));
+	strlcpy(principal.instance,
+			default_principal.instance,
+			sizeof(principal.instance));
+	strlcpy(principal.realm,
+			default_principal.realm,
+			sizeof(principal.realm));
     } else {
 	if (!principal.name[0])
-	    strcpy(principal.name, default_principal.name);
+	    strlcpy(principal.name,
+			    default_principal.name,
+			    sizeof(principal.name));
 	if (!principal.realm[0])
-	    strcpy(principal.realm, default_principal.realm);
+	    strlcpy(principal.realm,
+			    default_principal.realm,
+			    sizeof(principal.realm));
     }
 
-    sprintf(tktstring, TKT_ROOT "_cpw_%d",(int)getpid());
+    snprintf(tktstring, sizeof(tktstring), "%s_cpw_%u",
+	     TKT_ROOT, (unsigned)getpid());
     krb_set_tkt_string(tktstring);
     
     if (get_pw_new_pwd(pword, sizeof(pword), &principal,
@@ -154,7 +158,7 @@ main(int argc, char **argv)
 	memset(newkey, 0, sizeof(newkey));
       
 	if (status == KADM_INSECURE_PW)
-	    fprintf(stderr, "%s: Insecure password: %s\n", argv[0], pw_msg);
+	    warnx ("Insecure password: %s", pw_msg);
 	else if (status != KADM_SUCCESS)
 	    com_err(argv[0], status, " attempting to change password.");
     }
