@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997, 1998 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the 
  *    documentation and/or other materials provided with the distribution. 
  *
- * 3. All advertising materials mentioning features or use of this software 
- *    must display the following acknowledgement: 
- *      This product includes software developed by Kungliga Tekniska 
- *      Hgskolan and its contributors. 
- *
- * 4. Neither the name of the Institute nor the names of its contributors 
+ * 3. Neither the name of the Institute nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software 
  *    without specific prior written permission. 
  *
@@ -38,26 +33,29 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: com_err.c,v 1.7 1998/05/02 20:12:59 assar Exp $");
+RCSID("$Id: com_err.c,v 1.14 1999/12/02 16:58:37 joda Exp $");
 #endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <roken.h>
 #include "com_err.h"
 
 struct et_list *_et_list;
 
+
 const char *
 error_message (long code)
 {
     static char msg[128];
     const char *p = com_right(_et_list, code);
-    if(p){
-	strncpy(msg, p, sizeof(msg));
-	msg[sizeof(msg)-1] = '\0';
-    } else{
-	snprintf(msg, sizeof(msg), "Unknown error %d", code);
-    }
+    if (p == NULL)
+	p = strerror(code);
+    if (p != NULL && *p != '\0') {
+	strncpy(msg, p, sizeof(msg) - 1);
+	msg[sizeof(msg) - 1] = 0;
+    } else 
+	sprintf(msg, "Unknown error %ld", code);
     return msg;
 }
 
@@ -71,27 +69,13 @@ init_error_table(const char **msgs, long base, int count)
 static void
 default_proc (const char *whoami, long code, const char *fmt, va_list args)
 {
-    char f[sizeof("%s: %s %s\r\n")] = "";
-    char *x;
-    const void *arg[3], **ap = arg;
-    
-    if(whoami) {
-	strcat(f, "%s: ");
-	*ap++ = whoami;
-    }
-    if(code) {
-	strcat(f, "%s ");
-	*ap++ = error_message(code);
-    }
-    if(fmt) {
-	strcat(f, "%s");
-	*ap++ = fmt;
-    }
-    strcat(f, "\r\n");
-    asprintf(&x, f, arg[0], arg[1], arg[2]);
-    vfprintf(stderr, x, args);
-    free(x);
-    fflush(stderr);
+    if (whoami)
+      fprintf(stderr, "%s: ", whoami);
+    if (code)
+      fprintf(stderr, "%s ", error_message(code));
+    if (fmt)
+      vfprintf(stderr, fmt, args);
+    fprintf(stderr, "\r\n");	/* ??? */
 }
 
 static errf com_err_hook = default_proc;
