@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -41,7 +36,7 @@
 #include <string.h>
 #include <histedit.h>
 
-RCSID("$Id: edit_compat.c,v 1.5 1998/01/03 21:26:50 joda Exp $");
+RCSID("$Id: edit_compat.c,v 1.7 1999/12/02 16:58:39 joda Exp $");
 
 void
 rl_reset_terminal(char *p)
@@ -61,21 +56,34 @@ static const char* ret_prompt(EditLine *e)
 
 static History *h;
 
+#ifdef H_SETSIZE
+#define EL_INIT_FOUR 1
+#else
+#ifdef H_SETMAXSIZE
+/* backwards compatibility */
+#define H_SETSIZE H_SETMAXSIZE
+#endif
+#endif
+
 char *
 readline(const char* prompt)
 {
     static EditLine *e;
-#ifdef H_SETMAXSIZE
+#ifdef H_SETSIZE
     HistEvent ev;
 #endif
     int count;
     char *ret;
     if(e == NULL){
+#ifdef EL_INIT_FOUR
+	e = el_init("", stdin, stdout, stderr);
+#else
 	e = el_init("", stdin, stdout);
+#endif
 	el_set(e, EL_PROMPT, ret_prompt);
 	h = history_init();
-#ifdef H_SETMAXSIZE
-	history(h, &ev, H_SETMAXSIZE, 25);
+#ifdef H_SETSIZE
+	history(h, &ev, H_SETSIZE, 25);
 #else
 	history(h, H_EVENT, 25);
 #endif
@@ -95,7 +103,7 @@ readline(const char* prompt)
 void
 add_history(char *p)
 {
-#ifdef H_SETMAXSIZE
+#ifdef H_SETSIZE
     HistEvent ev;
     history(h, &ev, H_ENTER, p);
 #else
