@@ -1,5 +1,5 @@
 /*
- * $Id: kdc.h,v 1.7 1996/09/28 23:41:07 assar Exp $
+ * $Id: kdc.h,v 1.8 1997/04/01 03:59:05 assar Exp $
  *
  * Copyright 1987, 1988 by the Massachusetts Institute of Technology. 
  *
@@ -12,11 +12,6 @@
 #ifndef KDC_DEFS
 #define KDC_DEFS
 
-#define S_AD_SZ		sizeof(struct sockaddr_in)
-
-#define TRUE		1
-#define FALSE		0
-
 /* Don't depend on this! */
 #ifndef MKEYFILE
 #if 0
