@@ -21,7 +21,7 @@ or implied warranty.
 
 #include "kdb_locl.h"
 
-RCSID("$Id: krb_dbm.c,v 1.33 1998/05/28 15:12:01 joda Exp $");
+RCSID("$Id: krb_dbm.c,v 1.37 1999/09/16 20:41:49 assar Exp $");
 
 #include <xdbm.h>
 
@@ -107,16 +107,18 @@ gen_dbsuffix(char *db_name, char *sfx)
 	sfx = ".ok";
 
     asprintf (&dbsuffix, "%s%s", db_name, sfx);
+    if (dbsuffix == NULL) {
+	fprintf (stderr, "gen_dbsuffix: out of memory\n");
+	exit(1);
+    }
     return dbsuffix;
 }
 
 static void
 decode_princ_key(datum *key, char *name, char *instance)
 {
-    strncpy(name, key->dptr, ANAME_SZ);
-    strncpy(instance, (char *)key->dptr + ANAME_SZ, INST_SZ);
-    name[ANAME_SZ - 1] = '\0';
-    instance[INST_SZ - 1] = '\0';
+    strlcpy (name, key->dptr, ANAME_SZ);
+    strlcpy (instance, (char *)key->dptr + ANAME_SZ, INST_SZ);
 }
 
 static void
@@ -475,7 +477,7 @@ kerb_db_delete_principal (char *name, char *inst)
 	kerb_db_init();
     
     for(try = 0; try < KERB_DB_MAX_RETRY; try++){
-	if((code = kerb_dbl_lock(KERB_DBL_SHARED)) != 0)
+	if((code = kerb_dbl_lock(KERB_DBL_EXCLUSIVE)) != 0)
 	    return -1;
 	
 	db = dbm_open(current_db_name, O_RDWR, 0600);
@@ -525,6 +527,8 @@ kerb_db_get_principal (char *name, char *inst, Principal *principal,
 	    return -1;
 
 	db = dbm_open(current_db_name, O_RDONLY, 0600);
+	if (db == NULL)
+	    return -1;
 
 	*more = 0;
 
@@ -717,7 +721,6 @@ delta_stat(DB_stat *a, DB_stat *b, DB_stat *c)
     c->n_put_stat = a->n_put_stat - b->n_put_stat;
 
     memcpy(b, a, sizeof(DB_stat));
-    return;
 }
 
 /*
@@ -749,6 +752,8 @@ kerb_db_iterate (k_iter_proc_t func, void *arg)
 	return code;
 
     db = dbm_open(current_db_name, O_RDONLY, 0600);
+    if (db == NULL)
+	return errno;
 
     for (key = dbm_firstkey (db); key.dptr != NULL; key = dbm_next(db, key)) {
 	contents = dbm_fetch (db, key);
