@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,14 +33,13 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: get_default_principal.c,v 1.8 1996/10/27 16:29:38 joda Exp $");
+RCSID("$Id: get_default_principal.c,v 1.14 1999/12/02 16:58:41 joda Exp $");
 
 int
 krb_get_default_principal(char *name, char *instance, char *realm)
 {
   char *file;
   int ret;
-
   char *p;
 
   if ((file = getenv("KRBTKFILE")) == NULL)
@@ -55,6 +49,10 @@ krb_get_default_principal(char *name, char *instance, char *realm)
   if(ret == KSUCCESS)
       return 0;
 
+  p = getenv("KRB4PRINCIPAL");
+  if(p && kname_parse(name, instance, realm, p) == KSUCCESS)
+      return 1;
+
 #ifdef HAVE_PWD_H
   {
     struct passwd *pw;
@@ -63,11 +61,11 @@ krb_get_default_principal(char *name, char *instance, char *realm)
       return -1;
     }
 
-    strcpy(name, pw->pw_name);
-    strcpy(instance, "");
+    strlcpy (name, pw->pw_name, ANAME_SZ);
+    strlcpy (instance, "", INST_SZ);
     krb_get_lrealm(realm, 1);
 
-    if(strcmp(name, "root") == 0){
+    if(strcmp(name, "root") == 0) {
       p = NULL;
 #if defined(HAVE_GETLOGIN) && !defined(POSIX_GETLOGIN)
       p = getlogin();
@@ -77,13 +75,13 @@ krb_get_default_principal(char *name, char *instance, char *realm)
       if(p == NULL)
 	p = getenv("LOGNAME");
       if(p){
-	strncpy (name, p, ANAME_SZ);
-	name[ANAME_SZ - 1] = '\0';
-	strcpy(instance, "root");
+	  strlcpy (name, p, ANAME_SZ);
+	  strlcpy (instance, "root", INST_SZ);
       }
     }
     return 1;
   }
-#endif
+#else
   return -1;
+#endif
 }
