@@ -21,9 +21,11 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Id: getrealm.c,v 1.31 1998/03/18 13:46:49 bg Exp $");
+RCSID("$Id: getrealm.c,v 1.36 1999/09/16 20:41:51 assar Exp $");
 
-#define MATCH_SUBDOMAINS        0
+#ifndef MATCH_SUBDOMAINS
+#define MATCH_SUBDOMAINS 0
+#endif
 
 /*
  * krb_realmofhost.
@@ -75,8 +77,7 @@ dns_find_realm(char *hostname, char *realm)
 	    struct resource_record *rr = r->head;
 	    while(rr){
 		if(rr->type == T_TXT){
-		    strncpy(realm, rr->u.txt, REALM_SZ);
-		    realm[REALM_SZ - 1] = 0;
+		    strlcpy(realm, rr->u.txt, REALM_SZ);
 		    dns_free_data(r);
 		    return level;
 		}
@@ -130,24 +131,24 @@ file_find_realm(const char *phost, const char *domain,
 	tmp_realm = tok;
 	if (strcasecmp(tmp_host, phost) == 0) {
 	    /* exact match of hostname, so return the realm */
-	    strncpy(ret_realm, tmp_realm, ret_realm_sz);
-	    ret_realm[ret_realm_sz - 1] = '\0';
+	    strlcpy(ret_realm, tmp_realm, ret_realm_sz);
 	    ret = 0;
 	    break;
 	}
 	if ((tmp_host[0] == '.') && domain) { 
 	    const char *cp = domain;
 	    do {
-		if(strcasecmp(tmp_host, domain) == 0){
+		if(strcasecmp(tmp_host, cp) == 0){
 		    /* domain match, save for later */ 
-		    strncpy(ret_realm, tmp_realm, ret_realm_sz);
-		    ret_realm[ret_realm_sz - 1] = '\0';
+		    strlcpy(ret_realm, tmp_realm, ret_realm_sz);
 		    ret = 0;
 		    break;
 		}
 		cp = strchr(cp + 1, '.');
 	    } while(MATCH_SUBDOMAINS && cp);
 	}
+	if (ret == 0)
+	    break;
     }
     fclose(trans_file);
     return ret;
@@ -173,8 +174,7 @@ krb_realmofhost(const char *host)
     if (domain) {
 	char *cp;
 	  
-	strncpy(ret_realm, &domain[1], REALM_SZ);
-	ret_realm[REALM_SZ - 1] = 0;
+	strlcpy(ret_realm, &domain[1], REALM_SZ);
 	/* Upper-case realm */
 	for (cp = ret_realm; *cp; cp++)
 	    *cp = toupper(*cp);
