@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: kdc_reply.c,v 1.10 1998/03/19 15:45:28 joda Exp $");
+RCSID("$Id: kdc_reply.c,v 1.12 1999/12/02 16:58:42 joda Exp $");
 
 static int little_endian; /* XXX ugly */
 
@@ -52,15 +47,15 @@ kdc_reply_cred(KTEXT cip, CREDENTIALS *cred)
     
     if(p + strlen((char*)p) > cip->dat + cip->length)
 	return INTK_BADPW;
-    p += krb_get_string(p, cred->service);
+    p += krb_get_string(p, cred->service, sizeof(cred->service));
     
     if(p + strlen((char*)p) > cip->dat + cip->length)
 	return INTK_BADPW;
-    p += krb_get_string(p, cred->instance);
+    p += krb_get_string(p, cred->instance, sizeof(cred->instance));
     
     if(p + strlen((char*)p) > cip->dat + cip->length)
 	return INTK_BADPW;
-    p += krb_get_string(p, cred->realm);
+    p += krb_get_string(p, cred->realm, sizeof(cred->realm));
     
     if(p + 3 > cip->dat + cip->length)
 	return INTK_BADPW;
