@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: krb_locl.h,v 1.46 1997/12/05 01:29:41 assar Exp $ */
+/* $Id: krb_locl.h,v 1.50 1999/12/02 16:58:42 joda Exp $ */
 
 #ifndef __krb_locl_h
 #define __krb_locl_h
@@ -119,6 +114,11 @@
 
 #ifdef SOCKS
 #include <socks.h>
+
+/* This doesn't belong here. */
+struct tm *localtime(const time_t *);
+struct hostent  *gethostbyname(const char *);
+
 #endif
 
 #include <roken.h>
@@ -131,11 +131,6 @@
 
 /* --- */
 
-/* Globals! */
-extern int krb_debug;
-extern int krb_ap_req_debug;
-extern int krb_dns_debug;
-
 /* Utils */
 int
 krb_name_to_name __P((
@@ -174,4 +169,7 @@ fixup_quad_cksum __P((
 	void *old_checksum,
 	int little));
 
+void
+krb_kdctimeofday __P((struct timeval *tv));
+
 #endif /*  __krb_locl_h */
