@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,12 +33,29 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: kuserok.c,v 1.22 1997/12/08 23:37:47 joda Exp $");
+RCSID("$Id: kuserok.c,v 1.25 1999/12/02 16:58:42 joda Exp $");
 
 #define OK 0
 #define NOTOK 1
 #define MAX_USERNAME 10
 
+/*
+ * Return OK if `r' is one of the local realms, else NOTOK
+ */
+
+static int
+is_local_realm (const char *r)
+{
+    char lrealm[REALM_SZ];
+    int n;
+
+    for (n = 1; krb_get_lrealm(lrealm, n) == KSUCCESS; ++n) {
+	if (strcmp (r, lrealm) == 0)
+	    return OK;
+    }
+    return NOTOK;
+}
+
 /* 
  * Given a Kerberos principal and a local username, determine whether
  * user is authorized to login according to the authorization file
@@ -83,7 +95,6 @@ int
 krb_kuserok(char *name, char *instance, char *realm, char *luser)
 {
     struct passwd *pwd;
-    char lrealm[REALM_SZ];
     FILE *f;
     char line[1024];
     char file[MaxPathLen];
@@ -92,13 +103,12 @@ krb_kuserok(char *name, char *instance, char *realm, char *luser)
     pwd = getpwnam(luser);
     if(pwd == NULL)
 	return NOTOK;
-    if(krb_get_lrealm(lrealm, 1))
-	return NOTOK;
-    if(pwd->pw_uid != 0 &&
-       strcmp(name, luser) == 0 &&
-       strcmp(instance, "") == 0 &&
-       strcmp(realm, lrealm) == 0)
+    if (pwd->pw_uid != 0
+	&& strcmp (name, luser) == 0
+	&& strcmp (instance, "") == 0
+	&& is_local_realm (realm) == OK)
 	return OK;
+
     snprintf(file, sizeof(file), "%s/.klogin", pwd->pw_dir);
 
     f = fopen(file, "r");
@@ -134,10 +144,15 @@ krb_kuserok(char *name, char *instance, char *realm, char *luser)
 	    continue;
 	if(strcmp(instance, finst))
 	    continue;
-	if(frealm[0] == 0)
-	    strcpy(frealm, lrealm);
-	if(strcmp(realm, frealm))
+#if 0 /* don't support principals without realm any longer */
+	if(frealm[0] == 0) {
+	    if (is_local_realm (realm) != OK)
+		continue;
+	} else
+#endif
+	if (strcmp (realm, frealm))
 	    continue;
+
 	fclose(f);
 	return OK;
     }
