@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -39,44 +34,14 @@
 #include "krb_locl.h"
 #include <klog.h>
 
-RCSID("$Id: logging.c,v 1.9 1996/11/17 20:47:46 joda Exp $");
+RCSID("$Id: logging.c,v 1.18 1999/12/02 16:58:42 joda Exp $");
 
-struct krb_log_facility{
+struct krb_log_facility {
     char filename[MaxPathLen]; 
     FILE *file; 
     krb_log_func_t func;
 };
 
-
-static int
-log_with_timestamp_and_nl(FILE *file, const char *format, va_list args)
-{
-    time_t now;
-    if(file == NULL)
-	return KFAILURE;
-    time(&now);
-    fputs(krb_stime(&now), file);
-    fputs(": ", file);
-    vfprintf(file, format, args);
-    fputs("\n", file);
-    fflush(file);
-    return KSUCCESS;
-}
-
-static int
-log_tty(FILE *f, const char *format,  va_list args)
-{
-    if (f != NULL && isatty(fileno(f)))
-	vfprintf(f, format, args);
-    return KSUCCESS;
-}
-
-static int
-log_not(FILE *f, const char *format,  va_list args)
-{
-    return KSUCCESS;
-}
-
 int
 krb_vlogger(struct krb_log_facility *f, const char *format, va_list args)
 {
@@ -117,109 +82,37 @@ krb_openlog(struct krb_log_facility *f,
 	    FILE *file,
 	    krb_log_func_t func)
 {
-    strcpy(f->filename, filename);
+    strlcpy(f->filename, filename, MaxPathLen);
     f->file = file;
     f->func = func;
     return KSUCCESS;
 }
 
-/* ------------------------------------------------------------
-   Compatibility functions from log.c
-   ------------------------------------------------------------ */
-
-/* static facilities for krb_log() and klog() */
-static struct krb_log_facility file_log[2];
-
-static int KRBLOG_inited[2] = { 0, 0 };	/* Yuk! */
-
-/*
- * set_logfile() changes the name of the file to which
- * messages are logged. If set_logfile() is not called,
- * the logfile defaults to KRBLOG, defined in "krb.h".
- */
-
-void
-set_logfile(char *filename)
-{
-    krb_openlog(&file_log[0], filename, NULL, log_with_timestamp_and_nl);
-    KRBLOG_inited[0] = 1;
-}
-
-/*
- * krb_log() is used to add entries to the logfile (see set_logfile()).
- *
- * The log entry consists of a timestamp and the given arguments
- * printed according to the given "format".
- *
- * The log file is opened and closed for each log entry.
- */
-
-void
-krb_log(const char *format, ...)
-{
-    va_list args;
-
-    if (!KRBLOG_inited[0])
-        set_logfile(KRBLOG);	/* Horrible default! */
-
-    va_start(args, format);
-    krb_vlogger(&file_log[0], format, args);
-    va_end(args);
-}
-
-/* ------------------------------------------------------------
-   Compatibility functions from klog.c
-   ------------------------------------------------------------ */
-
-/*
- * klog() is used to add entries to the logfile (see kset_logfile()
- * below).
- *
- * The log entry consists of a timestamp and the given arguments
- * printed according to the given "format" string.
- *
- * The log file is opened and closed for each log entry.
- *
- * CHANGE: the type is always ignored
- *
- * If the given log type "type" is unknown, or if the log file
- * cannot be opened, no entry is made to the log file.
- *
- * The return value is always a pointer to the formatted log
- * text string "logtxt".
- */
-
-static char logtxt[1024];
-
-char *
-klog(int type, const char *format, ...)
-{
-    va_list ap;
-
-    va_start(ap, format);
-    vsprintf(logtxt, format, ap);
-    va_end(ap);
-    
-    if(!KRBLOG_inited[1])
-	kset_logfile(KRBLOG);
-    krb_logger(&file_log[1], "%s", logtxt);
-    
-    return logtxt;
-}
-
-void
-kset_logfile(char *filename)
-{
-    krb_openlog(&file_log[1], filename, NULL, log_with_timestamp_and_nl);
-    KRBLOG_inited[1] = 1;
-}
-
-
 /* ------------------------------------------------------------
    Compatibility functions from warning.c
    ------------------------------------------------------------ */
 
-static struct krb_log_facility std_log = { "/dev/tty", stderr, log_not };
+static int
+log_tty(FILE *f, const char *format,  va_list args)
+{
+    if (f != NULL && isatty(fileno(f)))
+	vfprintf(f, format, args);
+    return KSUCCESS;
+}
+
+/* stderr */
+static struct krb_log_facility std_log = { "/dev/tty", NULL, log_tty };
+
+static void
+init_std_log (void)
+{
+  static int done = 0;
+
+  if (!done) {
+    std_log.file = stderr;
+    done = 1;
+  }
+}
 
 /*
  *
@@ -227,6 +120,7 @@ static struct krb_log_facility std_log = { "/dev/tty", stderr, log_not };
 void
 krb_set_warnfn (krb_warnfn_t newfunc)
 {
+    init_std_log ();
     std_log.func =  newfunc;
 }
 
@@ -236,6 +130,7 @@ krb_set_warnfn (krb_warnfn_t newfunc)
 krb_warnfn_t
 krb_get_warnfn (void)
 {
+    init_std_log ();
     return std_log.func;
 }
 
@@ -247,7 +142,94 @@ krb_warning (const char *format, ...)
 {
     va_list args;
     
+    init_std_log ();
     va_start(args, format);
     krb_vlogger(&std_log, format, args);
     va_end(args);
 }
+
+/* ------------------------------------------------------------
+   Compatibility functions from klog.c and log.c
+   ------------------------------------------------------------ */
+
+/*
+ * Used by kerberos and kadmind daemons and in libkrb (rd_req.c).
+ *
+ * By default they log to the kerberos server log-file (KRBLOG) to be
+ * backwards compatible.
+ */
+
+static int
+log_with_timestamp_and_nl(FILE *file, const char *format, va_list args)
+{
+    time_t now;
+    if(file == NULL)
+	return KFAILURE;
+    time(&now);
+    fputs(krb_stime(&now), file);
+    fputs(": ", file);
+    vfprintf(file, format, args);
+    fputs("\n", file);
+    fflush(file);
+    return KSUCCESS;
+}
+
+static struct krb_log_facility
+file_log = { KRBLOG, NULL, log_with_timestamp_and_nl };
+
+/*
+ * kset_logfile() changes the name of the file to which
+ * messages are logged.  If kset_logfile() is not called,
+ * the logfile defaults to KRBLOG, defined in "krb.h".
+ */
+
+void
+kset_logfile(char *filename)
+{
+    krb_openlog(&file_log, filename, NULL, log_with_timestamp_and_nl);
+}
+
+/*
+ * krb_log() and klog() is used to add entries to the logfile.
+ *
+ * The log entry consists of a timestamp and the given arguments
+ * printed according to the given "format" string.
+ *
+ * The log file is opened and closed for each log entry.
+ *
+ * If the given log type "type" is unknown, or if the log file
+ * cannot be opened, no entry is made to the log file.
+ *
+ * CHANGE: the type is always ignored
+ *
+ * The return value of klog() is always a pointer to the formatted log
+ * text string "logtxt".
+ */
+
+/* Used in kerberos.c only. */
+char *
+klog(int type, const char *format, ...)
+{
+    static char logtxt[1024];
+
+    va_list ap;
+
+    va_start(ap, format);
+    vsnprintf(logtxt, sizeof(logtxt), format, ap);
+    va_end(ap);
+    
+    krb_logger(&file_log, "%s", logtxt);
+    
+    return logtxt;
+}
+
+/* Used in kadmind and rd_req.c */
+void
+krb_log(const char *format, ...)
+{
+    va_list args;
+
+    va_start(args, format);
+    krb_vlogger(&file_log, format, args);
+    va_end(args);
+}
