@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: mk_safe.c,v 1.23 1998/02/19 19:14:02 joda Exp $");
+RCSID("$Id: mk_safe.c,v 1.25 1999/12/02 16:58:43 joda Exp $");
 
 /* application include files */
 #include "krb-archaeology.h"
@@ -89,12 +84,12 @@ krb_mk_safe(void *in, void *out, u_int32_t length, des_cblock *key,
     unsigned char *start;
     u_int32_t src_addr;
 
-    p += krb_put_int(KRB_PROT_VERSION, p, 1);
-    p += krb_put_int(AUTH_MSG_SAFE, p, 1);
+    p += krb_put_int(KRB_PROT_VERSION, p, 1, 1);
+    p += krb_put_int(AUTH_MSG_SAFE, p, 1, 1);
     
     start = p;
 
-    p += krb_put_int(length, p, 4);
+    p += krb_put_int(length, p, 4, 4);
 
     memcpy(p, in, length);
     p += length;
@@ -104,9 +99,9 @@ krb_mk_safe(void *in, void *out, u_int32_t length, des_cblock *key,
     *p++ = tv.tv_usec/5000; /* 5ms */
     
     src_addr = sender->sin_addr.s_addr;
-    p += krb_put_address(src_addr, p);
+    p += krb_put_address(src_addr, p, 4);
 
-    p += krb_put_int(lsb_time(tv.tv_sec, sender, receiver), p, 4);
+    p += krb_put_int(lsb_time(tv.tv_sec, sender, receiver), p, 4, 4);
 
     {
 	/* We are faking big endian mode, so we need to fix the
