@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: rd_req.c,v 1.21 1996/11/19 22:48:48 assar Exp $");
+RCSID("$Id: rd_req.c,v 1.27.2.1 1999/12/06 22:04:36 assar Exp $");
 
 static struct timeval t_local = { 0, 0 };
 
@@ -91,9 +86,9 @@ krb_set_key(void *key, int cvt)
     return KSUCCESS;
 #else /* Encrypt */
     if (cvt)
-        des_string_to_key(key, &ky);
+        des_string_to_key((char*)key, &ky);
     else
-        memcpy(ky, key, 8);
+        memcpy((char*)ky, key, 8);
     return(des_key_sched(&ky, serv_key));
 #endif /* NOENCRYPTION */
 }
@@ -198,7 +193,7 @@ krb_rd_req(KTEXT authent,	/* The received message */
 
     s_kvno = *p++;
 
-    p += krb_get_string(p, realm);
+    p += krb_get_string(p, realm, sizeof(realm));
 
     /*
      * If "fn" is NULL, key info should already be set; don't
@@ -210,16 +205,16 @@ krb_rd_req(KTEXT authent,	/* The received message */
      */
     if (fn && (strcmp(st_nam,service) || strcmp(st_inst,instance) ||
                strcmp(st_rlm,realm) || (st_kvno != s_kvno))) {
-        if (*fn == 0) fn = KEYFILE;
+        if (*fn == 0) fn = (char *)KEYFILE;
         st_kvno = s_kvno;
         if (read_service_key(service, instance, realm, s_kvno,
 			     fn, (char *)skey))
             return(RD_AP_UNDEC);
-        if ((status = krb_set_key(skey, 0)))
+        if ((status = krb_set_key((char*)skey, 0)))
 	    return(status);
-        strcpy(st_rlm, realm);
-        strcpy(st_nam, service);
-        strcpy(st_inst, instance);
+        strlcpy (st_rlm, realm, REALM_SZ);
+        strlcpy (st_nam, service, SNAME_SZ);
+        strlcpy (st_inst, instance, INST_SZ);
     }
 
     tkt->length = *p++;
@@ -294,8 +289,10 @@ krb_rd_req(KTEXT authent,	/* The received message */
     delta_t = abs((int)(tv.tv_sec - r_time_sec));
     if (delta_t > CLOCK_SKEW) {
         if (krb_ap_req_debug)
-            krb_log("Time out of range: %d - %d = %d",
-		  t_local.tv_sec,r_time_sec,delta_t);
+            krb_log("Time out of range: %lu - %lu = %lu",
+		    (unsigned long)t_local.tv_sec,
+		    (unsigned long)r_time_sec,
+		    (unsigned long)delta_t);
         return RD_AP_TIME;
     }
 
@@ -303,8 +300,11 @@ krb_rd_req(KTEXT authent,	/* The received message */
 
     tkt_age = tv.tv_sec - ad->time_sec;
     if (krb_ap_req_debug)
-        krb_log("Time: %d Issue Date: %d Diff: %d Life %x",
-	      tv.tv_sec, ad->time_sec, tkt_age, ad->life);
+        krb_log("Time: %ld Issue Date: %lu Diff: %ld Life %x",
+		(long)tv.tv_sec,
+		(unsigned long)ad->time_sec,
+		tkt_age,
+		ad->life);
     
     if ((tkt_age < 0) && (-tkt_age > CLOCK_SKEW))
 	return RD_AP_NYV;
