@@ -21,7 +21,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Id: sendauth.c,v 1.16 1997/12/05 01:57:31 assar Exp $");
+RCSID("$Id: sendauth.c,v 1.18 1999/09/16 20:41:55 assar Exp $");
 
 /*
  * krb_sendauth() transmits a ticket over a file descriptor for a
@@ -148,8 +148,7 @@ krb_sendauth(int32_t options,	/* bit-pattern of options */
 	    i = instance;
 	else
 	    i = krb_get_phost(instance);
-	strncpy (inst, i, sizeof(inst));
-	inst[sizeof(inst) - 1] = '\0';
+	strlcpy (inst, i, sizeof(inst));
 
 	ret = krb_get_cred (service, inst, realm, cred);
 	if (ret != KSUCCESS)
