@@ -21,7 +21,7 @@ or implied warranty.
         
 #include "krb_locl.h"
 
-RCSID("$Id: tf_util.c,v 1.32 1998/05/28 15:13:31 joda Exp $");
+RCSID("$Id: tf_util.c,v 1.39 1999/12/02 18:03:16 assar Exp $");
 
 
 #define TOO_BIG -1
@@ -34,6 +34,10 @@ RCSID("$Id: tf_util.c,v 1.32 1998/05/28 15:13:31 joda Exp $");
 #define O_BINARY 0
 #endif
 
+#define MAGIC_TICKET_NAME "magic"
+#define MAGIC_TICKET_TIME_DIFF_INST "time-diff"
+#define MAGIC_TICKET_ADDR_INST "our-address"
+
 /*
  * fd must be initialized to something that won't ever occur as a real
  * file descriptor. Since open(2) returns only non-negative numbers as
@@ -162,12 +166,17 @@ tf_init(char *tf_name, int rw)
    * All library functions now assume that the right set of userids
    * are set upon entry, therefore it's not strictly necessary to
    * perform these test for programs adhering to these assumptions.
+   *
+   * This doesn't work on cygwin because getuid() returns a different
+   * uid than the owner of files that are created.
    */
+#ifndef __CYGWIN__
   {
     uid_t me = getuid();
     if (stat_buf.st_uid != me && me != 0)
       return TKT_FIL_ACC;
   }
+#endif
 
   /*
    * If "wflag" is set, open the ticket file in append-writeonly mode
@@ -303,7 +312,7 @@ tf_get_pname(char *p)
  */
 
 int
-tf_put_pname(char *p)
+tf_put_pname(const char *p)
 {
   unsigned count;
 
@@ -351,7 +360,7 @@ tf_get_pinst(char *inst)
  */
 
 int
-tf_put_pinst(char *inst)
+tf_put_pinst(const char *inst)
 {
   unsigned count;
 
@@ -377,11 +386,8 @@ tf_put_pinst(char *inst)
  * EOF          - end of file encountered
  */
 
-#define MAGIC_TICKET_NAME "magic"
-#define MAGIC_TICKET_INST "time-diff"
-	  
-int
-tf_get_cred(CREDENTIALS *c)
+static int
+real_tf_get_cred(CREDENTIALS *c)
 {
   KTEXT   ticket = &c->ticket_st;	/* pointer to ticket */
   int     k_errno;
@@ -391,7 +397,6 @@ tf_get_cred(CREDENTIALS *c)
       krb_warning ("tf_get_cred called before tf_init.\n");
     return TKT_FIL_INI;
   }
-again:
   if ((k_errno = tf_gets(c->service, SNAME_SZ)) < 2)
     switch (k_errno) {
     case TOO_BIG:
@@ -443,19 +448,71 @@ again:
       krb_warning ("tf_get_cred: failed tf_read.\n");
     return TKT_FIL_FMT;
   }
-  if(strcmp(c->service, MAGIC_TICKET_NAME) == 0 &&
-     strcmp(c->instance, MAGIC_TICKET_INST) == 0) {
-      /* we found the magic `time diff' ticket; update the kdc time
-         differential, and then get the next ticket */
-      u_int32_t d;
-
-      krb_get_int(c->ticket_st.dat, &d, 4, 0);
-      krb_set_kdc_time_diff(d);
-      goto again;
-  }
   return KSUCCESS;
 }
 
+int
+tf_get_cred(CREDENTIALS *c)
+{
+  int ret;
+  int fake;
+
+  do {
+    fake = 0;
+
+    ret = real_tf_get_cred (c);
+    if (ret)
+      return ret;
+
+    if(strcmp(c->service, MAGIC_TICKET_NAME) == 0) {
+      if(strcmp(c->instance, MAGIC_TICKET_TIME_DIFF_INST) == 0) {
+	/* we found the magic `time diff' ticket; update the kdc time
+         differential, and then get the next ticket */
+	u_int32_t d;
+
+	krb_get_int(c->ticket_st.dat, &d, 4, 0);
+	krb_set_kdc_time_diff(d);
+	fake = 1;
+      } else if (strcmp(c->instance, MAGIC_TICKET_ADDR_INST) == 0) {
+	fake = 1;
+      }
+    }
+  } while (fake);
+  return ret;
+}
+
+int
+tf_get_cred_addr(char *realm, size_t realm_sz, struct in_addr *addr)
+{
+  int ret;
+  int fake;
+  CREDENTIALS cred;
+
+  do {
+    fake = 1;
+
+    ret = real_tf_get_cred (&cred);
+    if (ret)
+      return ret;
+
+    if(strcmp(cred.service, MAGIC_TICKET_NAME) == 0) {
+      if(strcmp(cred.instance, MAGIC_TICKET_TIME_DIFF_INST) == 0) {
+	/* we found the magic `time diff' ticket; update the kdc time
+         differential, and then get the next ticket */
+	u_int32_t d;
+
+	krb_get_int(cred.ticket_st.dat, &d, 4, 0);
+	krb_set_kdc_time_diff(d);
+      } else if (strcmp(cred.instance, MAGIC_TICKET_ADDR_INST) == 0) {
+	strlcpy(realm, cred.realm, realm_sz);
+	memcpy (addr, cred.ticket_st.dat, sizeof(*addr));
+	fake = 0;
+      }
+    }
+  } while (fake);
+  return ret;
+}
+
 /*
  * tf_close() closes the ticket file and sets "fd" to -1. If "fd" is
  * not a valid file descriptor, it just returns.  It also clears the
@@ -627,7 +684,7 @@ bad:
 }
 
 int
-tf_setup(CREDENTIALS *cred, char *pname, char *pinst)
+tf_setup(CREDENTIALS *cred, const char *pname, const char *pinst)
 {
     int ret;
     ret = tf_create(tkt_string());
@@ -645,13 +702,13 @@ tf_setup(CREDENTIALS *cred, char *pname, char *pinst)
            to the kdc. The first ticket defines which realm we belong
            to, but since this ticket gets the same realm as the tgt,
            this shouldn't be a problem */
-	CREDENTIALS magic;
 	des_cblock s = { 0, 0, 0, 0, 0, 0, 0, 0 };
 	KTEXT_ST t;
 	int d = krb_get_kdc_time_diff();
-	krb_put_int(d, t.dat, 4);
+	krb_put_int(d, t.dat, sizeof(t.dat), 4);
 	t.length = 4;
-	tf_save_cred(MAGIC_TICKET_NAME, MAGIC_TICKET_INST, cred->realm, s, 
+	tf_save_cred(MAGIC_TICKET_NAME, MAGIC_TICKET_TIME_DIFF_INST,
+		     cred->realm, s, 
 		     cred->lifetime, 0, &t, cred->issue_date);
     }
     ret = tf_save_cred(cred->service, cred->instance, cred->realm, 
@@ -679,3 +736,70 @@ in_tkt(char *pname, char *pinst)
     tf_close();
     return KSUCCESS;
 }
+
+/*
+ * If there's a magic ticket with an address for realm `realm' in
+ * ticket file, return it in `addr'.
+ * realm == NULL means any realm.
+ */
+
+int
+tf_get_addr (const char *realm, struct in_addr *addr)
+{
+  CREDENTIALS cred;
+  krb_principal princ;
+  int ret;
+
+  ret = tf_init (tkt_string (), R_TKT_FIL);
+  if (ret)
+    return ret;
+
+  ret = tf_get_pname (princ.name);
+  if (ret)
+    goto out;
+  ret = tf_get_pinst (princ.name);
+  if (ret)
+    goto out;
+  while ((ret = real_tf_get_cred (&cred)) == KSUCCESS) {
+    if (strcmp (cred.service, MAGIC_TICKET_NAME) == 0
+	&& strcmp (cred.instance, MAGIC_TICKET_ADDR_INST) == 0
+	&& (realm == NULL
+	    || strcmp (cred.realm, realm) == 0)) {
+      memcpy (addr, cred.ticket_st.dat, sizeof(*addr));
+      goto out;
+    }
+  }
+  ret = KFAILURE;
+
+out:
+  tf_close ();
+  return ret;
+}
+
+/*
+ * Store `realm, addr' as a magic ticket.
+ */
+
+int
+tf_store_addr (const char *realm, struct in_addr *addr)
+{
+  CREDENTIALS c;
+  krb_principal princ;
+  int ret;
+  des_cblock s = { 0, 0, 0, 0, 0, 0, 0, 0 };
+  KTEXT_ST t;
+
+  ret = tf_init (tkt_string (), W_TKT_FIL);
+  if (ret)
+    return ret;
+
+  t.length = sizeof(*addr);
+  memcpy (t.dat, addr, sizeof(*addr));
+
+  ret = tf_save_cred (MAGIC_TICKET_NAME, MAGIC_TICKET_ADDR_INST,
+		      (char *)realm, s, 0, /* lifetime */ 
+		      0, /* kvno */
+		      &t, time(NULL));
+  tf_close ();
+  return ret;
+}
