@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,22 +33,38 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: verify_user.c,v 1.9 1997/12/05 01:30:31 assar Exp $");
+RCSID("$Id: verify_user.c,v 1.17.2.1 1999/12/06 22:57:17 assar Exp $");
 
-/* Verify user with password. If secure, also verify against local
- * service key, this can (usually) only be done by root.
+/*
+ * Verify user (name.instance@realm) with `password'.
+ *
+ * If secure, also verify against local
+ * service key (`linstance'.hostname) (or rcmd if linstance == NULL),
+ * this can (usually) only be done by root.
+ *
+ * If secure == KRB_VERIFY_SECURE, fail if there's no key.
+ * If secure == KRB_VERIFY_SECURE_FAIL, don't fail if there's no such
+ * key in the srvtab.
  *
  * As a side effect, fresh tickets are obtained.
  *
+ * srvtab is where the key is found.
+ * 
  * Returns zero if ok, a positive kerberos error or -1 for system
  * errors.
  */
 
-int
-krb_verify_user(char *name, char *instance, char *realm, char *password, 
-		int secure, char *linstance)
+static int
+krb_verify_user_srvtab_exact(char *name,
+			     char *instance,
+			     char *realm,
+			     char *password, 
+			     int secure,
+			     char *linstance,
+			     char *srvtab)
 {
     int ret;
+
     ret = krb_get_pw_in_tkt(name, instance, realm,
 			    KRB_TICKET_GRANTING_TICKET,
 			    realm,
@@ -61,7 +72,7 @@ krb_verify_user(char *name, char *instance, char *realm, char *password,
     if(ret != KSUCCESS)
 	return ret;
 
-    if(secure){
+    if(secure == KRB_VERIFY_SECURE || secure == KRB_VERIFY_SECURE_FAIL){
 	struct hostent *hp;
 	int32_t addr;
 	
@@ -94,13 +105,21 @@ krb_verify_user(char *name, char *instance, char *realm, char *password,
 	if (linstance == NULL)
 	    linstance = "rcmd";
 
+	if(secure == KRB_VERIFY_SECURE_FAIL) {
+	    des_cblock key;
+	    ret = read_service_key(linstance, phost, lrealm, 0, srvtab, &key);
+	    memset(key, 0, sizeof(key));
+	    if(ret == KFAILURE)
+		return 0;
+	}
+	
 	ret = krb_mk_req(&ticket, linstance, phost, lrealm, 33);
 	if(ret != KSUCCESS){
 	    dest_tkt();
 	    return ret;
 	}
 	
-	ret = krb_rd_req(&ticket, linstance, phost, addr, &auth, "");
+	ret = krb_rd_req(&ticket, linstance, phost, addr, &auth, srvtab);
 	if(ret != KSUCCESS){
 	    dest_tkt();
 	    return ret;
@@ -109,3 +128,58 @@ krb_verify_user(char *name, char *instance, char *realm, char *password,
     return 0;
 }
 		
+/*
+ * Try to verify the user and password against all the local realms.
+ */
+
+int
+krb_verify_user_srvtab(char *name,
+		       char *instance,
+		       char *realm,
+		       char *password, 
+		       int secure,
+		       char *linstance,
+		       char *srvtab)
+{
+  int ret;
+  int n;
+  char rlm[256];
+
+  /* First try to verify against the supplied realm. */
+  ret = krb_verify_user_srvtab_exact(name, instance, realm, password,
+				     secure, linstance, srvtab);
+  if (ret == KSUCCESS)
+    return KSUCCESS;
+
+  /* Verify all local realms, except the supplied realm. */
+  for (n = 1; krb_get_lrealm(rlm, n) == KSUCCESS; n++)
+    if (strcmp(rlm, realm) != 0) {
+      ret = krb_verify_user_srvtab_exact(name, instance, rlm, password,
+					 secure, linstance, srvtab);
+      if (ret == KSUCCESS)
+	return KSUCCESS;
+    }
+
+  return ret;
+}
+
+/*
+ * Compat function without srvtab.
+ */
+
+int
+krb_verify_user(char *name,
+		char *instance,
+		char *realm,
+		char *password, 
+		int secure,
+		char *linstance)
+{
+    return krb_verify_user_srvtab (name,
+				   instance,
+				   realm,
+				   password,
+				   secure,
+				   linstance,
+				   (char *)KEYFILE);
+}
