@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #include "rxkad_locl.h"
 
-RCSID("$Id: rxk_info.c,v 1.7 1998/05/12 14:54:53 bg Exp $");
+RCSID("$Id: rxk_info.c,v 1.8 1999/12/02 16:58:55 joda Exp $");
 
 int32
 rxkad_GetServerInfo(struct rx_connection *con,
